# -*- coding: utf-8 -*-

import unittest
import os
import re

try:
    from mock import patch, MagicMock
except ImportError:
    from unittest.mock import patch, MagicMock

from .context import pydatajson
from pydatajson.federation import *
from ckanapi.errors import NotFound

SAMPLES_DIR = os.path.join("tests", "samples")


class PushDatasetTestCase(unittest.TestCase):

    @classmethod
    def get_sample(cls, sample_filename):
        return os.path.join(SAMPLES_DIR, sample_filename)

    @classmethod
    def setUpClass(cls):
        cls.catalog = pydatajson.DataJson(cls.get_sample('full_data.json'))
        cls.catalog_id = cls.catalog.get('identifier', re.sub(
            r'[^a-z-_]+', '', cls.catalog['title']).lower())
        cls.dataset = cls.catalog.datasets[0]
        cls.dataset_id = cls.dataset['identifier']
        cls.minimum_catalog = pydatajson.DataJson(
            cls.get_sample('minimum_data.json'))
        cls.minimum_catalog_id = cls.minimum_catalog.get('identifier',
                                                         re.sub(r'[^a-z-_]+', '', cls.minimum_catalog['title']).lower())
        cls.minimum_dataset = cls.minimum_catalog.datasets[0]
        # PATCH: minimum_data no tiene los identificadores obligatorios. Se los agrego aca, pero hay que refactorizar
        # tests y samples desactualizados para cumplir con los perfiles nuevos
        cls.minimum_dataset['identifier'] = cls.dataset['identifier']
        cls.minimum_dataset['distribution'][0][
            'identifier'] = cls.dataset['distribution'][0]['identifier']

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_id_is_created_correctly(self, mock_portal):
        def mock_call_action(action, data_dict=None):
            if action == 'package_update':
                raise NotFound
            if action == 'package_create':
                return data_dict
            else:
                return []
        mock_portal.return_value.call_action = mock_call_action
        res_id = push_dataset_to_ckan(self.catalog, 'owner', self.dataset_id,
                                      'portal', 'key', catalog_id=self.catalog_id)
        self.assertEqual(self.catalog_id + '_' + self.dataset_id, res_id)

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_id_is_updated_correctly(self, mock_portal):
        def mock_call_action(action, data_dict=None):
            if action == 'package_update':
                return data_dict
            if action == 'package_create':
                self.fail('should not be called')
            else:
                return []
        mock_portal.return_value.call_action = mock_call_action
        res_id = push_dataset_to_ckan(self.catalog, 'owner', self.dataset_id,
                                      'portal', 'key', catalog_id=self.catalog_id)
        self.assertEqual(self.catalog_id + '_' + self.dataset_id, res_id)

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_dataset_id_is_preserved_if_catalog_id_is_not_passed(self, mock_portal):
        def mock_call_action(action, data_dict=None):
            if action == 'package_update':
                return data_dict
            if action == 'package_create':
                self.fail('should not be called')
            else:
                return []

        mock_portal.return_value.call_action = mock_call_action
        res_id = push_dataset_to_ckan(self.catalog, 'owner', self.dataset_id,
                                      'portal', 'key')
        self.assertEqual(self.dataset_id, res_id)

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_tags_are_passed_correctly(self, mock_portal):
        themes = self.dataset['theme']
        keywords = [kw for kw in self.dataset['keyword']]
        for theme in themes:
            label = self.catalog.get_theme(identifier=theme)['label']
            label = re.sub(r'[^\w .-]+', '', label, flags=re.UNICODE)
            keywords.append(label)

        def mock_call_action(action, data_dict=None):
            if action == 'package_update':
                try:
                    self.assertItemsEqual(keywords, [tag['name'] for tag in data_dict['tags']])
                except AttributeError:
                    self.assertCountEqual(keywords, [tag['name'] for tag in data_dict['tags']])
                return data_dict
            if action == 'package_create':
                self.fail('should not be called')
            else:
                return []

        mock_portal.return_value.call_action = mock_call_action
        res_id = push_dataset_to_ckan(self.catalog, 'owner', self.dataset_id,
                                      'portal', 'key', catalog_id=self.catalog_id)
        self.assertEqual(self.catalog_id + '_' + self.dataset_id, res_id)

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_licenses_are_interpreted_correctly(self, mock_portal):
        def mock_call_action(action, data_dict=None):
            if action == 'license_list':
                return [{'title': 'somelicense', 'url': 'somelicense.com', 'id': '1'},
                        {'title': 'otherlicense', 'url': 'otherlicense.com', 'id': '2'}]
            elif action == 'package_update':
                self.assertEqual('notspecified', data_dict['license_id'])
                return data_dict
            else:
                return []
        mock_portal.return_value.call_action = mock_call_action
        push_dataset_to_ckan(self.catalog, 'owner', self.dataset_id,
                             'portal', 'key', catalog_id=self.catalog_id)

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_dataset_without_license_sets_notspecified(self, mock_portal):
        def mock_call_action(action, data_dict=None):
            if action == 'license_list':
                return [{'title': 'somelicense', 'url': 'somelicense.com', 'id': '1'},
                        {'title': 'otherlicense', 'url': 'otherlicense.com', 'id': '2'}]
            elif action == 'package_update':
                self.assertEqual('notspecified', data_dict['license_id'])
                return data_dict
            else:
                return []

        mock_portal.return_value.call_action = mock_call_action
        push_dataset_to_ckan(self.minimum_catalog, 'owner', self.minimum_dataset['identifier'],
                             'portal', 'key', catalog_id=self.minimum_catalog_id)

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_dataset_level_wrappers(self, mock_portal):
        def mock_call_action(action, data_dict=None):
            if action == 'package_update':
                return data_dict
            else:
                return []
        mock_portal.return_value.call_action = mock_call_action
        restored_id = restore_dataset_to_ckan(self.catalog, 'owner', self.dataset_id,
                                              'portal', 'key')
        harvested_id = harvest_dataset_to_ckan(self.catalog, 'owner', self.dataset_id,
                                               'portal', 'key', self.catalog_id)
        self.assertEqual(self.dataset_id, restored_id)
        self.assertEqual(self.catalog_id+'_'+self.dataset_id, harvested_id)
    
    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_harvest_catalog_with_no_optional_parametres(self, mock_portal):
        def mock_call_action(action, data_dict=None):
            if action == 'package_update':
                self.assertTrue(data_dict['id'].startswith(self.catalog_id+'_'))
                self.assertTrue(data_dict['name'].startswith(self.catalog_id+'-'))
                self.assertEqual(self.catalog_id, data_dict['owner_org'])
                return data_dict
            else:
                return []
        mock_portal.return_value.call_action = mock_call_action
        harvested_ids = harvest_catalog_to_ckan(self.catalog, 'portal', 'key', self.catalog_id)
        try:
            self.assertItemsEqual([self.catalog_id+'_'+ds['identifier'] for ds in self.catalog.datasets],
                                  harvested_ids)
        except AttributeError:
            self.assertCountEqual([self.catalog_id+'_'+ds['identifier'] for ds in self.catalog.datasets],
                                  harvested_ids)

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_harvest_catalog_with_dataset_list(self, mock_portal):
        def mock_call_action(action, data_dict=None):
            if action == 'package_update':
                return data_dict
            else:
                return []

        mock_portal.return_value.call_action = mock_call_action

        dataset_list = [ds['identifier'] for ds in self.catalog.datasets[:1]]
        harvested_ids = harvest_catalog_to_ckan(self.catalog, 'portal', 'key', self.catalog_id,
                                                dataset_list=dataset_list)
        try:
            self.assertItemsEqual([self.catalog_id+'_'+ds_id for ds_id in dataset_list],
                                  harvested_ids)
        except AttributeError:
            self.assertCountEqual([self.catalog_id+'_'+ds_id for ds_id in dataset_list],
                                  harvested_ids)

        dataset_list = [ds['identifier'] for ds in self.catalog.datasets]
        harvested_ids = harvest_catalog_to_ckan(self.catalog, 'portal', 'key', self.catalog_id,
                                                dataset_list=dataset_list)
        try:
            self.assertItemsEqual([self.catalog_id+'_'+ds_id for ds_id in dataset_list],
                                  harvested_ids)
        except AttributeError:
            self.assertCountEqual([self.catalog_id+'_'+ds_id for ds_id in dataset_list],
                                  harvested_ids)

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_harvest_catalog_with_owner_org(self, mock_portal):
        def mock_call_action(action, data_dict=None):
            if action == 'package_update':
                self.assertEqual('owner', data_dict['owner_org'])
                return data_dict
            else:
                return []

        mock_portal.return_value.call_action = mock_call_action
        harvested_ids = harvest_catalog_to_ckan(self.catalog, 'portal', 'key', self.catalog_id,
                                                owner_org='owner')
        try:
            self.assertItemsEqual([self.catalog_id+'_'+ds['identifier'] for ds in self.catalog.datasets],
                                  harvested_ids)
        except AttributeError:
            self.assertCountEqual([self.catalog_id+'_'+ds['identifier'] for ds in self.catalog.datasets],
                                  harvested_ids)


class RemoveDatasetTestCase(unittest.TestCase):

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_empty_search_doesnt_call_purge(self, mock_portal):
        mock_portal.return_value.call_action = MagicMock()
        remove_datasets_from_ckan('portal', 'key')
        mock_portal.return_value.call_action.assert_not_called()

    @patch('pydatajson.federation.get_datasets')
    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_filter_in_datasets(self, mock_portal, mock_search):
        mock_portal.return_value.call_action = MagicMock()
        mock_search.return_value = ['some_id']
        filter_in = {'dataset': {'id': 'some_id'}}
        remove_datasets_from_ckan('portal', 'key',  filter_in=filter_in)
        mock_portal.return_value.call_action.assert_called_with(
            'dataset_purge', data_dict={'id': 'some_id'})

    @patch('pydatajson.federation.get_datasets')
    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_filter_in_out_datasets(self, mock_portal, mock_search):
        mock_portal.return_value.call_action = MagicMock()
        mock_search.return_value = ['some_id', 'other_id']
        filter_out = {'dataset': {'id': 'some_id'}}
        remove_datasets_from_ckan('portal', 'key', filter_out=filter_out)
        mock_portal.return_value.call_action.assert_any_call(
            'dataset_purge', data_dict={'id': 'other_id'})
        mock_portal.return_value.call_action.assert_any_call(
            'dataset_purge', data_dict={'id': 'some_id'})

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_query_one_dataset(self, mock_portal):
        mock_portal.return_value.call_action = MagicMock(
            return_value={'count': 1, 'results': [{'id': 'some_id'}]})
        remove_datasets_from_ckan('portal', 'key',  organization='some_org')
        data_dict = {'q': 'organization:"some_org"', 'rows': 500, 'start': 0}
        mock_portal.return_value.call_action.assert_any_call(
            'package_search', data_dict=data_dict)
        mock_portal.return_value.call_action.assert_any_call(
            'dataset_purge', data_dict={'id': 'some_id'})

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_query_over_500_datasets(self, mock_portal):
        count = 1001
        # First, the query results. Then the "dataset_purge" None results
        side_effects = [{'count': count, 'results': [{'id': 'id_1'}]},
                        {'count': count, 'results': [{'id': 'id_2'}]},
                        {'count': count, 'results': [{'id': 'id_3'}]},
                        None, None, None
                        ]
        mock_portal.return_value.call_action = MagicMock(
            side_effect=side_effects)
        remove_datasets_from_ckan('portal', 'key', organization='some_org')
        for start in range(0, count, 500):
            data_dict = {'q': 'organization:"some_org"',
                         'rows': 500, 'start': start}
            mock_portal.return_value.call_action.assert_any_call(
                'package_search', data_dict=data_dict)
        for x in ['1', '2', '3']:
            mock_portal.return_value.call_action.assert_any_call(
                'dataset_purge', data_dict={'id': 'id_' + x})

    @patch('pydatajson.federation.get_datasets')
    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_remove_through_filters_and_organization(self, mock_portal, mock_search):
        filter_results = ['id_1', 'id_2']
        org_results = [{'id': 'id_2'}, {'id': 'id_3'}]
        mock_search.return_value = filter_results
        mock_portal.return_value.call_action = MagicMock(
            return_value={'count': 2, 'results': org_results})
        remove_datasets_from_ckan(
            'portal', 'key', only_time_series=True, organization='some_org')
        mock_portal.return_value.call_action.assert_called_with(
            'dataset_purge', data_dict={'id': 'id_2'})


class PushThemeTestCase(unittest.TestCase):

    @classmethod
    def get_sample(cls, sample_filename):
        return os.path.join(SAMPLES_DIR, sample_filename)

    @classmethod
    def setUpClass(cls):
        cls.catalog = pydatajson.DataJson(cls.get_sample('full_data.json'))

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_empty_theme_search_raises_exception(self, mock_portal):
        with self.assertRaises(AssertionError):
            push_theme_to_ckan(self.catalog, 'portal_url', 'apikey')

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_function_pushes_theme_by_identifier(self, mock_portal):
        mock_portal.return_value.call_action = MagicMock(return_value={'name': 'group_name'})
        result = push_theme_to_ckan(self.catalog, 'portal_url', 'apikey', identifier='compras')
        self.assertEqual('group_name', result)

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_function_pushes_theme_by_label(self, mock_portal):
        mock_portal.return_value.call_action = MagicMock(return_value={'name': 'other_name'})
        result = push_theme_to_ckan(self.catalog, 'portal_url', 'apikey', label='Adjudicaciones')
        self.assertEqual('other_name', result)

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_ckan_portal_is_called_with_correct_parametres(self, mock_portal):
        mock_portal.return_value.call_action = MagicMock(return_value={'name': u'contrataciones'})
        group = {'name': u'contrataciones',
                 'title': u'Contrataciones',
                 'description': u'Datasets sobre contrataciones.'}
        push_theme_to_ckan(self.catalog, 'portal_url', 'apikey', identifier='contrataciones')
        mock_portal.return_value.call_action.assert_called_once_with('group_create', data_dict=group)


class PushCatalogThemesTestCase(unittest.TestCase):
    @classmethod
    def get_sample(cls, sample_filename):
        return os.path.join(SAMPLES_DIR, sample_filename)

    @classmethod
    def setUpClass(cls):
        cls.catalog = pydatajson.DataJson(cls.get_sample('full_data.json'))

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_empty_portal_pushes_every_theme(self, mock_portal):
        def mock_call_action(action, data_dict=None):
            if action == 'group_list':
                return []
            elif action == 'group_create':
                return {'name': data_dict['name']}
            else:
                self.fail('should not be called')

        mock_portal.return_value.call_action = mock_call_action
        res_names = push_new_themes(self.catalog, 'portal_url', 'apikey')
        try:
            self.assertItemsEqual([theme['id'] for theme in self.catalog['themeTaxonomy']], res_names)
        except AttributeError:
            self.assertCountEqual([theme['id'] for theme in self.catalog['themeTaxonomy']], res_names)

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_full_portal_pushes_nothing(self, mock_portal):
        def mock_call_action(action, data_dict=None):
            if action == 'group_list':
                return [theme['id'] for theme in self.catalog['themeTaxonomy']]
            else:
                self.fail('should not be called')

        mock_portal.return_value.call_action = mock_call_action
        res_names = push_new_themes(self.catalog, 'portal_url', 'apikey')
        try:
            self.assertItemsEqual([], res_names)
        except AttributeError:
            self.assertCountEqual([], res_names)

    @patch('pydatajson.federation.RemoteCKAN', autospec=True)
    def test_non_empty_intersection_pushes_missing_themes(self, mock_portal):
        def mock_call_action(action, data_dict=None):
            if action == 'group_list':
                return [theme['id'] for theme in self.catalog['themeTaxonomy']][0:2]
            elif action == 'group_create':
                return {'name': data_dict['name']}
            else:
                self.fail('should not be called')

        mock_portal.return_value.call_action = mock_call_action
        res_names = push_new_themes(self.catalog, 'portal_url', 'apikey')
        try:
            self.assertItemsEqual([theme['id'] for theme in self.catalog['themeTaxonomy']][2:], res_names)
        except AttributeError:
            self.assertCountEqual([theme['id'] for theme in self.catalog['themeTaxonomy']][2:], res_names)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
import re
from datetime import time
from dateutil import parser, tz
from .helpers import title_to_name


def append_attribute_to_extra(package, dataset, attribute, serialize=False):
    value = dataset.get(attribute)
    if value:
        if serialize:
            value = json.dumps(value)
        package['extras'].append({'key': attribute, 'value': value})


def map_dataset_to_package(catalog, dataset, owner_org, catalog_id=None,
                           demote_superThemes=True, demote_themes=True):
    package = dict()
    package['extras'] = []
#   Obligatorios
    package['id'] = catalog_id+'_'+dataset['identifier'] if catalog_id else dataset['identifier']
    package['name'] = title_to_name(catalog_id+'-'+dataset['title'] if catalog_id else dataset['title'])
    package['title'] = dataset['title']
    package['private'] = False
    package['notes'] = dataset['description']
    package['author'] = dataset['publisher']['name']
    package['owner_org'] = owner_org

    append_attribute_to_extra(package, dataset, 'issued')
    append_attribute_to_extra(package, dataset, 'accrualPeriodicity')

    distributions = dataset['distribution']
    package['resources'] = map_distributions_to_resources(distributions, catalog_id)

    super_themes = dataset['superTheme']
    append_attribute_to_extra(package, dataset, 'superTheme', serialize=True)
    if demote_superThemes:
        package['groups'] = [{'name': title_to_name(super_theme, decode=False)} for super_theme in super_themes]


#   Recomendados y opcionales
    package['url'] = dataset.get('landingPage')
    package['author_email'] = dataset['publisher'].get('mbox')
    append_attribute_to_extra(package, dataset, 'modified')
    append_attribute_to_extra(package, dataset, 'temporal')
    append_attribute_to_extra(package, dataset, 'source')
    append_attribute_to_extra(package, dataset, 'language', serialize=True)

    spatial = dataset.get('spatial')
    if spatial:
        serializable = type(spatial) is list
        append_attribute_to_extra(package, dataset, 'spatial', serializable)

    contact_point = dataset.get('contactPoint')
    if contact_point:
        package['maintainer'] = contact_point.get('fn')
        package['maintainer_email'] = contact_point.get('hasEmail')

    keywords = dataset.get('keyword')
    if keywords:
        package['tags'] = [{'name': keyword} for keyword in keywords]

    # Move themes to keywords
    themes = dataset.get('theme', [])
    if themes and demote_themes:
        package['tags'] = package.get('tags', [])
        for theme in themes:
            label = catalog.get_theme(identifier=theme)['label']
            label = re.sub(r'[^\wá-úÁ-ÚñÑ .-]+', '', label, flags=re.UNICODE)
            package['tags'].append({'name': label})
    else:
        package['groups'] = package.get('groups', []) + [{'name': title_to_name(theme, decode=False)}
                                                         for theme in themes]

    return package


def convert_iso_string_to_utc(date_string):
    date_time = parser.parse(date_string)
    if date_time.time() == time(0):
        return date_string

    if date_time.tzinfo is not None:
        utc_date_time = date_time.astimezone(tz.tzutc())
    else:
        utc_date_time = date_time
    utc_date_time = utc_date_time.replace(tzinfo=None)
    return utc_date_time.isoformat()


def map_distributions_to_resources(distributions, catalog_id=None):
    resources = []
    for distribution in distributions:
        resource = dict()
#       Obligatorios
        resource['id'] = catalog_id + '_' + distribution['identifier'] if catalog_id else distribution['identifier']
        resource['name'] = distribution['title']
        resource['url'] = distribution['downloadURL']
        resource['created'] = convert_iso_string_to_utc(distribution['issued'])
#       Recomendados y opcionales
        resource['description'] = distribution.get('description')
        resource['format'] = distribution.get('format')
        last_modified = distribution.get('modified')
        if last_modified:
            resource['last_modified'] = convert_iso_string_to_utc(last_modified)
        resource['mimetype'] = distribution.get('mediaType')
        resource['size'] = distribution.get('byteSize')
        resource['accessURL'] = distribution.get('accessURL')
        fileName = distribution.get('fileName')
        if fileName:
            resource['fileName'] = fileName
        dist_fields = distribution.get('field')
        if dist_fields:
            resource['attributesDescription'] = json.dumps(dist_fields)
        resources.append(resource)

    return resources


def map_theme_to_group(theme):

    return {
        "name": title_to_name(theme.get('id') or theme['label']),
        "title": theme.get('label'),
        "description": theme.get('description'),
    }



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Módulo 'readers' de Pydatajson

Contiene los métodos auxiliares para leer archivos con información tabular y
catálogos de metadatos, en distintos fomatos.
"""

from __future__ import unicode_literals, print_function
from __future__ import with_statement, absolute_import

import io
import json
import logging
import os.path
import warnings

import openpyxl as pyxl
import requests
import unicodecsv as csv
from six import string_types, text_type, iteritems
from six.moves.urllib_parse import urlparse
from unidecode import unidecode

import pydatajson
from . import custom_exceptions as ce
from . import helpers

import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

global_logger = logging.getLogger()


def read_catalog_obj(catalog):
    try:
        if getattr(catalog, "has_catalog"):
            return catalog
        else:
            return pydatajson.DataJson(catalog)
    except:
        return pydatajson.DataJson(catalog)


def read_catalog(catalog, default_values=None):
    """Toma una representación cualquiera de un catálogo, y devuelve su
    representación interna (un diccionario de Python con su metadata.)

    Si recibe una representación _interna_ (un diccionario), lo devuelve
    intacto. Si recibe una representación _externa_ (path/URL a un archivo
    JSON/XLSX), devuelve su represetación interna, es decir, un diccionario.

    Args:
        catalog (dict or str): Representación externa/interna de un catálogo.
            Una representación _externa_ es un path local o una URL remota a un
            archivo con la metadata de un catálogo, en formato JSON o XLSX. La
            representación _interna_ de un catálogo es un diccionario.

    Returns:
        dict: Representación interna de un catálogo para uso en las funciones
        de esta librería.
    """
    unknown_catalog_repr_msg = """
No se pudo inferir una representación válida de un catálogo del parámetro
provisto: {}.""".format(catalog)
    assert isinstance(catalog, string_types + (dict,)
                      ), unknown_catalog_repr_msg

    if isinstance(catalog, dict):
        catalog_dict = catalog

    else:
        # catalog es una URL remota o un path local
        suffix = catalog.split(".")[-1].strip("/")
        unknown_suffix_msg = """
{} no es un sufijo conocido. Pruebe con 'json' o  'xlsx'""".format(suffix)
        assert suffix in ["json", "xlsx"], unknown_suffix_msg

        if suffix == "json":
            catalog_dict = read_json(catalog)
        elif suffix == "xlsx":
            # El archivo está en formato XLSX
            catalog_dict = read_xlsx_catalog(catalog)
        else:
            raise Exception("Formato no reconocido {}".format(suffix))
    # si se pasaron valores default, los aplica al catálogo leído
    if default_values:
        apply_default_values(catalog_dict, default_values)

    return catalog_dict


def apply_default_values(catalog, default_values):
    """Aplica valores default a los campos de un catálogo.

    Si el campo está vacío, aplica el default. Si tiene un valor, deja el valor
    que estaba. Sólo soporta defaults para las siguientes clases:
        catalog
        dataset
        distribution
        field

    Args:
        catalog (dict): Un catálogo.
        default_values (dict): Valores default para algunos de los campos del
            catálogo.
            {
                "dataset_issued": "2017-06-22",
                "distribution_issued": "2017-06-22"
            }
    """

    for field, default_value in iteritems(default_values):
        class_metadata = field.split("_")[0]
        field_json_path = field.split("_")[1:]

        # valores default de catálogo
        if class_metadata == "catalog":
            _set_default_value(catalog, field_json_path, default_value)

        # valores default de dataset
        elif class_metadata == "dataset":
            for dataset in catalog["dataset"]:
                _set_default_value(dataset, field_json_path, default_value)

        # valores default de distribución
        elif class_metadata == "distribution":
            for dataset in catalog["dataset"]:
                for distribution in dataset["distribution"]:
                    _set_default_value(
                        distribution, field_json_path, default_value)

        # valores default de field
        elif class_metadata == "field":
            for dataset in catalog["dataset"]:
                for distribution in dataset["distribution"]:

                    # campo "field" en una "distribution" no es obligatorio
                    if distribution.get("field"):
                        for field in distribution["field"]:
                            _set_default_value(
                                field, field_json_path, default_value)


def _set_default_value(dict_obj, keys, value):
    """Setea valor en diccionario anidado, siguiendo lista de keys.

    Args:
        dict_obj (dict): Un diccionario anidado.
        keys (list): Una lista de keys para navegar el diccionario.
        value (any): Un valor para reemplazar.
    """
    variable = dict_obj

    if len(keys) == 1:
        if not variable.get(keys[0]):
            variable[keys[0]] = value

    else:
        for idx, field in enumerate(keys):
            if idx < len(keys) - 1:
                variable = variable[field]

        if not variable.get(keys[-1]):
            variable[keys[-1]] = value


def read_json(json_path_or_url):
    """Toma el path a un JSON y devuelve el diccionario que representa.

    Se asume que el parámetro es una URL si comienza con 'http' o 'https', o
    un path local de lo contrario.

    Args:
        json_path_or_url (str): Path local o URL remota a un archivo de texto
            plano en formato JSON.

    Returns:
        dict: El diccionario que resulta de deserializar json_path_or_url.

    """
    assert isinstance(json_path_or_url, string_types)

    parsed_url = urlparse(json_path_or_url)
    if parsed_url.scheme in ["http", "https"]:
        res = requests.get(json_path_or_url, verify=False)
        json_dict = json.loads(res.content, encoding='utf-8')

    else:
        # Si json_path_or_url parece ser una URL remota, lo advierto.
        path_start = parsed_url.path.split(".")[0]
        if path_start == "www" or path_start.isdigit():
            warnings.warn("""
La dirección del archivo JSON ingresada parece una URL, pero no comienza
con 'http' o 'https' así que será tratada como una dirección local. ¿Tal vez
quiso decir 'http://{}'?""".format(json_path_or_url).encode("utf-8"))

        with io.open(json_path_or_url, encoding='utf-8') as json_file:
            json_dict = json.load(json_file)

    return json_dict


def read_xlsx_catalog(xlsx_path_or_url, logger=None):
    """Toma el path a un catálogo en formato XLSX y devuelve el diccionario
    que representa.

    Se asume que el parámetro es una URL si comienza con 'http' o 'https', o
    un path local de lo contrario.

    Args:
        xlsx_path_or_url (str): Path local o URL remota a un libro XLSX de
            formato específico para guardar los metadatos de un catálogo.

    Returns:
        dict: El diccionario que resulta de procesar xlsx_path_or_url.

    """
    logger = logger or global_logger
    assert isinstance(xlsx_path_or_url, string_types)

    parsed_url = urlparse(xlsx_path_or_url)
    if parsed_url.scheme in ["http", "https"]:
        res = requests.get(xlsx_path_or_url, verify=False)
        tmpfilename = ".tmpfile.xlsx"
        with io.open(tmpfilename, 'wb') as tmpfile:
            tmpfile.write(res.content)
        catalog_dict = read_local_xlsx_catalog(tmpfilename, logger)
        os.remove(tmpfilename)
    else:
        # Si xlsx_path_or_url parece ser una URL remota, lo advierto.
        path_start = parsed_url.path.split(".")[0]
        if path_start == "www" or path_start.isdigit():
            warnings.warn("""
La dirección del archivo JSON ingresada parece una URL, pero no comienza
con 'http' o 'https' así que será tratada como una dirección local. ¿Tal vez
quiso decir 'http://{}'?
            """.format(xlsx_path_or_url).encode("utf8"))

        catalog_dict = read_local_xlsx_catalog(xlsx_path_or_url)

    return catalog_dict


def _make_publisher(catalog_or_dataset):
    """De estar presentes las claves necesarias, genera el diccionario
    "publisher" a nivel catálogo o dataset."""
    level = catalog_or_dataset
    keys = [k for k in ["publisher_name", "publisher_mbox"] if k in level]
    if keys:
        level["publisher"] = {
            key.replace("publisher_", ""): level.pop(key) for key in keys
        }
    return level


def _make_contact_point(dataset):
    """De estar presentes las claves necesarias, genera el diccionario
    "contactPoint" de un dataset."""
    keys = [k for k in ["contactPoint_fn", "contactPoint_hasEmail"]
            if k in dataset]
    if keys:
        dataset["contactPoint"] = {
            key.replace("contactPoint_", ""): dataset.pop(key) for key in keys
        }
    return dataset


def _get_dataset_index(catalog, dataset_identifier, dataset_title,
                       logger=None):
    """Devuelve el índice de un dataset en el catálogo en función de su
    identificador"""
    logger = logger or global_logger
    matching_datasets = []

    for idx, dataset in enumerate(catalog["catalog_dataset"]):
        if dataset["dataset_identifier"] == dataset_identifier:
            if dataset["dataset_title"] == dataset_title:
                matching_datasets.append(idx)
            else:
                logger.warning(
                    ce.DatasetUnexpectedTitle(
                        dataset_identifier,
                        dataset["dataset_title"],
                        dataset_title
                    )
                )

    # Debe haber exactamente un dataset con el identificador provisto.
    no_dsets_msg = "No hay ningun dataset con el identifier {}".format(
        dataset_identifier)
    many_dsets_msg = "Hay mas de un dataset con el identifier {}: {}".format(
        dataset_identifier, matching_datasets)
    if len(matching_datasets) == 0:
        print(no_dsets_msg)
        return None
    elif len(matching_datasets) > 1:
        print(many_dsets_msg)
        return None
    else:
        return matching_datasets[0]


def _get_distribution_indexes(catalog, dataset_identifier, dataset_title,
                              distribution_identifier, distribution_title,
                              logger=None):
    """Devuelve el índice de una distribución en su dataset en función de su
    título, junto con el índice de su dataset padre en el catálogo, en
    función de su identificador"""
    logger = logger or global_logger
    dataset_index = _get_dataset_index(
        catalog, dataset_identifier, dataset_title)
    if dataset_index is None:
        return None, None
    else:
        dataset = catalog["catalog_dataset"][dataset_index]

    matching_distributions = []

    for idx, distribution in enumerate(dataset["dataset_distribution"]):
        if distribution["distribution_identifier"] == distribution_identifier:
            if distribution["distribution_title"] == distribution_title:
                matching_distributions.append(idx)
            else:
                logger.warning(
                    ce.DistributionUnexpectedTitle(
                        distribution_identifier,
                        distribution["distribution_title"],
                        distribution_title
                    )
                )

    # Debe haber exactamente una distribución con los identicadores provistos
    if len(matching_distributions) == 0:
        logger.warning(
            ce.DistributionTitleNonExistentError(
                distribution_title, dataset_identifier
            )
        )
        return dataset_index, None
    elif len(matching_distributions) > 1:
        logger.warning(
            ce.DistributionTitleRepetitionError(
                distribution_title, dataset_identifier, matching_distributions)
        )
        return dataset_index, None
    else:
        distribution_index = matching_distributions[0]
        return dataset_index, distribution_index


def read_local_xlsx_catalog(xlsx_path, logger=None):
    """Genera un diccionario de metadatos de catálogo a partir de un XLSX bien
    formado.

    Args:
        xlsx_path (str): Path a un archivo XLSX "template" para describir la
            metadata de un catálogo.

    Returns:
        dict: Diccionario con los metadatos de un catálogo.
    """
    logger = logger or global_logger
    assert xlsx_path.endswith(".xlsx"), """
El archivo a leer debe tener extensión XLSX."""

    wb = pyxl.load_workbook(xlsx_path, data_only=True, read_only=True)

    # Toma las hojas del modelo, resistente a mayúsuculas/minúsculas
    ws_catalog = helpers.get_ws_case_insensitive(wb, "catalog")
    ws_dataset = helpers.get_ws_case_insensitive(wb, "dataset")
    ws_distribution = helpers.get_ws_case_insensitive(wb, "distribution")
    ws_theme = helpers.get_ws_case_insensitive(wb, "theme")
    ws_field = helpers.get_ws_case_insensitive(wb, "field")

    catalogs = helpers.sheet_to_table(ws_catalog)
    # Debe haber exactamente un catálogo en la hoja 'Catalog'
    assert (len(catalogs) != 0), "No hay ningun catálogo en la hoja 'Catalog'"
    assert (len(catalogs) < 2), "Hay mas de un catálogo en la hoja 'Catalog'"
    # Genero el catálogo base
    catalog = catalogs[0]

    # Agrego themes y datasets al catálogo
    catalog["catalog_dataset"] = helpers.sheet_to_table(ws_dataset)

    # Me aseguro que los identificadores de dataset se guarden como cadenas
    for dataset in catalog["catalog_dataset"]:
        dataset["dataset_identifier"] = text_type(
            dataset["dataset_identifier"])

    catalog["catalog_themeTaxonomy"] = (
        helpers.sheet_to_table(ws_theme))

    # Agrego lista de distribuciones vacía a cada dataset
    for dataset in catalog["catalog_dataset"]:
        dataset["dataset_distribution"] = []

    # Ubico cada distribución en su dataset
    distributions = helpers.sheet_to_table(ws_distribution)
    for distribution in distributions:
        # Me aseguro que los identificadores se guarden como cadenas
        distribution["dataset_identifier"] = text_type(
            distribution["dataset_identifier"])
        distribution["distribution_identifier"] = text_type(
            distribution["distribution_identifier"])

        dataset_index = _get_dataset_index(
            catalog, distribution["dataset_identifier"],
            distribution["dataset_title"], logger)
        if dataset_index is None:
            print("""La distribucion con ID '{}' y titulo '{}' no se
pudo asignar a un dataset, y no figurara en el data.json de salida.""".format(
                distribution["distribution_identifier"],
                distribution["distribution_title"]))
        else:
            dataset = catalog["catalog_dataset"][dataset_index]
            dataset["dataset_distribution"].append(distribution)

    # Ubico cada campo en su distribución
    fields = helpers.sheet_to_table(ws_field)
    for idx, field in enumerate(fields):
        # Me aseguro que los identificadores se guarden como cadenas
        field["dataset_identifier"] = text_type(field["dataset_identifier"])
        field["distribution_identifier"] = text_type(
            field["distribution_identifier"])

        dataset_index, distribution_index = _get_distribution_indexes(
            catalog, field["dataset_identifier"], field["dataset_title"],
            field["distribution_identifier"], field["distribution_title"],
            logger)

        if dataset_index is None:
            print("""No se encontro el dataset '{}' especificado para el campo
'{}' (fila #{} de la hoja "Field"). Este campo no figurara en el data.json de salida.""".format(
                unidecode(field["dataset_title"]),
                unidecode(field["field_title"]),
                idx + 2))

        elif distribution_index is None:
            print("""No se encontro la distribucion '{}' especificada para el campo
'{}' (fila #{} de la hoja "Field"). Este campo no figurara en el data.json de salida.""".format(
                unidecode(field["distribution_title"]),
                unidecode(field["field_title"]),
                idx + 2))

        else:
            dataset = catalog["catalog_dataset"][dataset_index]
            distribution = dataset["dataset_distribution"][distribution_index]

            if "distribution_field" in distribution:
                distribution["distribution_field"].append(field)
            else:
                distribution["distribution_field"] = [field]

    # Transformo campos de texto separado por comas en listas
    if "catalog_language" in catalog:
        catalog["catalog_language"] = helpers.string_to_list(
            catalog["catalog_language"])

    for dataset in catalog["catalog_dataset"]:
        array_fields = ["dataset_superTheme", "dataset_theme", "dataset_tags",
                        "dataset_keyword", "dataset_language"]
        for field in array_fields:
            if field in dataset:
                dataset[field] = helpers.string_to_list(dataset[field])

    # Elimino los prefijos de los campos a nivel catálogo
    for old_key in list(catalog.keys()):
        if old_key.startswith("catalog_"):
            new_key = old_key.replace("catalog_", "")
            catalog[new_key] = catalog.pop(old_key)
        else:
            catalog.pop(old_key)

    # Elimino los prefijos de los campos a nivel tema
    for theme in catalog["themeTaxonomy"]:
        for old_key in theme.keys():
            if old_key.startswith("theme_"):
                new_key = old_key.replace("theme_", "")
                theme[new_key] = theme.pop(old_key)
            else:
                theme.pop(old_key)

    # Elimino los prefijos de los campos a nivel dataset
    for dataset in catalog["dataset"]:
        for old_key in list(dataset.keys()):
            if old_key.startswith("dataset_"):
                new_key = old_key.replace("dataset_", "")
                dataset[new_key] = dataset.pop(old_key)
            else:
                dataset.pop(old_key)

    # Elimino los campos auxiliares y los prefijos de los campos a nivel
    # distribución
    for dataset in catalog["dataset"]:
        for distribution in dataset["distribution"]:
            for old_key in list(distribution.keys()):
                if old_key.startswith("distribution_"):
                    new_key = old_key.replace("distribution_", "")
                    distribution[new_key] = distribution.pop(old_key)
                else:
                    distribution.pop(old_key)

    # Elimino campos auxiliares y los prefijos de los campos a nivel "campo"
    for dataset in catalog["dataset"]:
        for distribution in dataset["distribution"]:
            if "field" in distribution:
                for field in distribution["field"]:
                    for old_key in list(field.keys()):
                        if old_key.startswith("field_"):
                            new_key = old_key.replace("field_", "")
                            field[new_key] = field.pop(old_key)
                        else:
                            field.pop(old_key)

    # Agrupo las claves de "publisher" y "contactPoint" en sendos diccionarios
    catalog = _make_publisher(catalog)
    for dataset in catalog["dataset"]:
        dataset = _make_publisher(dataset)
        dataset = _make_contact_point(dataset)

    return catalog


def read_table(path):
    """Lee un archivo tabular (CSV o XLSX) a una lista de diccionarios.

    La extensión del archivo debe ser ".csv" o ".xlsx". En función de
    ella se decidirá el método a usar para leerlo.

    Si recibe una lista, comprueba que todos sus diccionarios tengan las
    mismas claves y de ser así, la devuelve intacta. Levanta una Excepción
    en caso contrario.

    Args:
        path(str o list): Como 'str', path a un archivo CSV o XLSX.

    Returns:
        list: Lista de diccionarios con claves idénticas representando el
        archivo original.
    """
    assert isinstance(path, string_types + (list,)), """
{} no es un `path` valido""".format(path)

    # Si `path` es una lista, devolverla intacta si tiene formato tabular.
    # Si no, levantar una excepción.
    if isinstance(path, list):
        if helpers.is_list_of_matching_dicts(path):
            return path
        else:
            raise ValueError("""
La lista ingresada no esta formada por diccionarios con las mismas claves.""")

    # Deduzco el formato de archivo de `path` y redirijo según corresponda.
    suffix = path.split(".")[-1]
    if suffix == "csv":
        return _read_csv_table(path)
    elif suffix == "xlsx":
        return _read_xlsx_table(path)
    else:
        raise ValueError("""
{} no es un sufijo reconocido. Pruebe con .csv o .xlsx""".format(suffix))


def _read_csv_table(path):
    """Lee un CSV a una lista de diccionarios."""
    with open(path, 'rb') as csvfile:
        reader = csv.DictReader(csvfile)
        table = list(reader)
    return table


def _read_xlsx_table(path):
    """Lee la hoja activa de un archivo XLSX a una lista de diccionarios."""
    workbook = pyxl.load_workbook(path)
    worksheet = workbook.active
    table = helpers.sheet_to_table(worksheet)

    return table



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys
sys.path.insert(0, os.path.abspath("."))
sys.path.insert(0, os.path.abspath(".."))

import pydatajson
import pydatajson.search
import pydatajson.backup



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
from collections import OrderedDict

import six


def to_native_dict(ordered_dict):
    return json.loads(json.dumps(ordered_dict))


def to_dict(table_list):
    ordered_dict = OrderedDict(table_list)
    if six.PY3:
        return to_native_dict(ordered_dict)
    else:
        return ordered_dict


CSV_TABLE = [
    to_dict([(u'Plato', u'Milanesa'),
             (u'Precio', u'Bajo'),
             (u'Sabor', u'666')]),
    to_dict([(u'Plato', u'Thoné, Vitel'),
             (u'Precio', u'Alto'),
             (u'Sabor', u'8000')]),
    to_dict([(u'Plato', u'Aceitunas'),
             (u'Precio', u''),
             (u'Sabor', u'15')])
]

WRITE_XLSX_TABLE = [
    to_dict([(u'Plato', u'Milanesa'),
             (u'Precio', u'Bajo'),
             (u'Sabor', 666)]),
    to_dict([(u'Plato', u'Thoné, Vitel'),
             (u'Precio', u'Alto'),
             (u'Sabor', 8000)]),
    to_dict([(u'Plato', u'Aceitunas'),
             (u'Precio', None),
             (u'Sabor', 15)])
]

READ_XLSX_TABLE = [
    to_dict([(u'Plato', u'Milanesa'),
             (u'Precio', u'Bajo'),
             (u'Sabor', 666)]),
    to_dict([(u'Plato', u'Thoné, Vitel'),
             (u'Precio', u'Alto'),
             (u'Sabor', 8000)]),
    to_dict([(u'Plato', u'Aceitunas'),
             (u'Sabor', 15)])
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests para las funcionalidades de el módulo 'backup'
"""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import with_statement

from functools import wraps
import unittest
import nose
import os
import io
import json
import vcr

from six import text_type

from .context import pydatajson

SAMPLES_DIR = os.path.join("tests", "samples")
RESULTS_DIR = os.path.join("tests", "results")

VCR = vcr.VCR(path_transformer=vcr.VCR.ensure_suffix('.yaml'),
              cassette_library_dir=os.path.join("tests", "cassetes", "backup"),
              record_mode='once')


class BackupTestCase(unittest.TestCase):
    """Tests for backup methods."""

    @classmethod
    def get_sample(cls, sample_filename):
        return os.path.join(SAMPLES_DIR, sample_filename)

    @classmethod
    def setUp(cls):
        cls.catalog_meta = pydatajson.DataJson(
            cls.get_sample("full_data.json"))
        cls.catalog_data = pydatajson.DataJson(
            cls.get_sample("example_time_series.json"))
        cls.maxDiff = None
        cls.longMessage = True

    @classmethod
    def tearDown(cls):
        del (cls.catalog_meta)
        del (cls.catalog_data)

    def test_make_catalog_backup_metadata(self):
        json_path = os.path.join(
            RESULTS_DIR, "catalog", "example", "data.json")
        xlsx_path = os.path.join(
            RESULTS_DIR, "catalog", "example", "catalog.xlsx")

        os.remove(json_path) if os.path.exists(json_path) else None
        os.remove(xlsx_path) if os.path.exists(xlsx_path) else None

        pydatajson.backup.make_catalog_backup(
            self.catalog_meta,
            catalog_id="example", local_catalogs_dir=RESULTS_DIR,
            include_metadata=True, include_data=False)

        self.assertTrue(os.path.exists(json_path))
        self.assertTrue(os.path.exists(xlsx_path))

    @VCR.use_cassette()
    def test_make_catalog_backup_data(self):
        distribution_path = os.path.abspath(os.path.join(
            RESULTS_DIR, "catalog", "example_ts", "dataset", "1",
            "distribution", "1.2", "download",
            "oferta-demanda-globales-datos-desestacionalizados-valores-trimestrales-base-1993.csv"
        ))

        os.remove(distribution_path) if os.path.exists(
            distribution_path) else None

        pydatajson.backup.make_catalog_backup(
            self.catalog_data,
            catalog_id="example_ts", local_catalogs_dir=RESULTS_DIR,
            include_metadata=True, include_data=True)

        print(distribution_path)
        self.assertTrue(os.path.exists(distribution_path))

    @VCR.use_cassette()
    def test_make_catalog_backup_data_without_file_name(self):
        distribution_path = os.path.abspath(os.path.join(
            RESULTS_DIR, "catalog", "example_ts", "dataset", "1",
            "distribution", "1.2.b", "download",
            "odg-total-millones-pesos-1960-trimestral.csv"
        ))

        os.remove(distribution_path) if os.path.exists(
            distribution_path) else None

        pydatajson.backup.make_catalog_backup(
            self.catalog_data,
            catalog_id="example_ts", local_catalogs_dir=RESULTS_DIR,
            include_metadata=True, include_data=True)

        print(distribution_path)
        self.assertTrue(os.path.exists(distribution_path))


if __name__ == '__main__':
    nose.run(defaultTest=__name__)



# -*- coding: utf-8 -*-


from __future__ import unicode_literals

from tests.support.utils import jsonschema_str


def dataset_error(options=None):
    default_options = {
        "dataset_title": "Sistema de contrataciones electrónicas",
        "instance": "RP1Y",
        "validator": "anyOf",
        "path": [
            "dataset",
            0,
            "accrualPeriodicity"
        ],
        "message": "u'RP1Y' is not valid under any of the given schemas",
        "error_code": 2,
        "validator_value": [
            {
                "pattern": "^R/P\\d+(\\.\\d+)?[Y|M|W|D]$",
                "type": "string"
            },
            {
                "pattern": "^R/PT\\d+(\\.\\d+)?[H|M|S]$",
                "type": "string"
            },
            {
                "pattern": "^eventual$",
                "type": "string"
            }
        ]
    }
    if options is not None:
        default_options.update(options)
    options = default_options
    return {
        "status": "ERROR",
        "error": {
            "catalog": {
                "status": "OK",
                "errors": [],
                "title": "Datos Argentina"
            },
            "dataset": [
                {
                    "status": "ERROR",
                    "identifier": "99db6631-d1c9-470b-a73e-c62daa32c420",
                    "list_index": 0,
                    "errors": [
                        {
                            "instance": options['instance'],
                            "validator": options['validator'],
                            "path": options['path'],
                            "message": options['message'],
                            "error_code": options['error_code'],
                            "validator_value": options['validator_value'],
                        }
                    ],
                    "title": options['dataset_title']
                }
            ]
        }
    }


def missing_dataset_title():
    return dataset_error({
        "instance": None,
        "validator": "required",
        "path": [
            "dataset",
            0
        ],
        "message": "%s is a required property" % jsonschema_str('title'),
        "error_code": 1,
        "validator_value": [
            "title",
            "description",
            "publisher",
            "superTheme",
            "distribution",
            "accrualPeriodicity",
            "issued"
        ],
        "dataset_title": None,
    })


def missing_dataset_description():
    return dataset_error({
        "instance": None,
        "validator": "required",
        "path": [
            "dataset",
            0
        ],
        "message": "%s is a required property" % jsonschema_str('description'),
        "error_code": 1,
        "validator_value": [
            "title",
            "description",
            "publisher",
            "superTheme",
            "distribution",
            "accrualPeriodicity",
            "issued"
        ],
        "dataset_title": 'Sistema de contrataciones electrónicas',
    })


def malformed_accrualperiodicity():
    return dataset_error({
        'message': "%s is not valid under any of the given schemas" % jsonschema_str('RP1Y'),
    })


def malformed_temporal():
    return dataset_error({
        "instance": "2015-01-1/2015-12-31",
        "validator": "anyOf",
        "path": [
            "dataset",
            0,
            "temporal"
        ],
        "message": "%s is not valid under any of the given schemas" % jsonschema_str(
            '2015-01-1/2015-12-31'),
        "error_code": 2,
        "validator_value": [
            {
                "pattern": "^(\\d{4}-\\d\\d-\\d\\d(T\\d\\d:\\d\\d:\\d\\d(\\.\\d+)?)?(([+-]\\d\\d:\\d\\d)|Z)?)\\/(\\d{4}-\\d\\d-\\d\\d(T\\d\\d:\\d\\d:\\d\\d(\\.\\d+)?)?(([+-]\\d\\d:\\d\\d)|Z)?)$",
                "type": "string"
            },
            {
                "type": "null"
            },
            {
                "type": "string",
                "maxLength": 0
            }
        ]
    })


def malformed_temporal2():
    return dataset_error({
        "instance": "2015-01-10/31-12-2015",
        "validator": "anyOf",
        "path": [
            "dataset",
            0,
            "temporal"
        ],
        "message": "%s is not valid under any of the given schemas" % jsonschema_str(
            '2015-01-10/31-12-2015'),
        "error_code": 2,
        "validator_value": [
            {
                "pattern": "^(\\d{4}-\\d\\d-\\d\\d(T\\d\\d:\\d\\d:\\d\\d(\\.\\d+)?)?(([+-]\\d\\d:\\d\\d)|Z)?)\\/(\\d{4}-\\d\\d-\\d\\d(T\\d\\d:\\d\\d:\\d\\d(\\.\\d+)?)?(([+-]\\d\\d:\\d\\d)|Z)?)$",
                "type": "string"
            },
            {
                "type": "null"
            },
            {
                "type": "string",
                "maxLength": 0
            }
        ]
    })


def too_long_field_title():
    return dataset_error({
        "instance": "organismo_unidad_operativa_contrataciones_desc_organismo_unidad_operativa_contrataciones_desc",
        "validator": "anyOf",
        "path": [
            "dataset",
            0,
            "distribution",
            0,
            "field",
            3,
            "title"
        ],
        "message": "%s is not valid under any of the given schemas" % jsonschema_str(
            'organismo_unidad_operativa_contrataciones_desc_organismo_unidad_operativa_contrataciones_desc'),
        "error_code": 2,
        "validator_value": [
            {
                "type": "string",
                "maxLength": 60
            },
            {
                "type": "null"
            }
        ]
    })



import unittest
import os
import vcr
from ckanapi import RemoteCKAN
from ckanapi.errors import NotFound
from pydatajson.helpers import title_to_name
from pydatajson.federation import push_dataset_to_ckan, remove_datasets_from_ckan
from .context import pydatajson

SAMPLES_DIR = os.path.join("tests", "samples")


class PushTestCase(unittest.TestCase):
    CKAN_VCR = vcr.VCR(path_transformer=vcr.VCR.ensure_suffix('.yaml'),
                       cassette_library_dir=os.path.join(
                           "tests", "cassetes", "ckan_integration", "push_dataset"),
                       filter_headers=['Authorization', 'X-CKAN-API-Key'],
                       record_mode='once')

    portal_url = 'http://localhost:8080'
    apikey = "<apikey>"

    @classmethod
    def get_sample(cls, sample_filename):
        return os.path.join(SAMPLES_DIR, sample_filename)

    @CKAN_VCR.use_cassette()
    def setUp(self):
        self.portal = RemoteCKAN(self.portal_url, apikey=self.apikey)
        self.full_catalog = pydatajson.DataJson(
            self.get_sample('full_data.json'))
        self.justice_catalog = pydatajson.DataJson(
            self.get_sample('catalogo_justicia.json'))

    @CKAN_VCR.use_cassette()
    def tearDown(self):
        full_dataset = self.full_catalog.datasets[0]
        full_name = title_to_name(full_dataset['title'])
        justice_dataset = self.justice_catalog.datasets[0]
        justice_name = title_to_name(justice_dataset['title'])
        try:
            self.portal.call_action(
                'dataset_purge', data_dict={'id': full_name})
        except NotFound:
            pass
        try:
            self.portal.call_action(
                'dataset_purge', data_dict={'id': justice_name})
        except NotFound:
            pass

        self.portal.close()

    @CKAN_VCR.use_cassette()
    def test_dataset_is_created_correctly(self):
        catalog = self.full_catalog
        catalog_id = title_to_name(catalog['title'])
        dataset = catalog.datasets[0]
        dataset_id = dataset['identifier']
        return_id = push_dataset_to_ckan(catalog, "oficina-de-muestra", dataset_id,
                                         self.portal_url, self.apikey, catalog_id=catalog_id,)
        self.assertEqual(return_id, catalog_id + '_' + dataset_id)

    @CKAN_VCR.use_cassette()
    def test_dataset_is_updated_correctly(self):
        catalog = self.full_catalog
        catalog_id = title_to_name(catalog['title'])
        dataset_id = catalog.datasets[0]['identifier']
        push_dataset_to_ckan(catalog, "oficina-de-muestra", dataset_id,
                             self.portal_url, self.apikey, catalog_id=catalog_id,)

        catalog.datasets[0]['description'] = 'updated description'
        return_id = push_dataset_to_ckan(catalog, "oficina-de-muestra", dataset_id,
                                         self.portal_url, self.apikey, catalog_id=catalog_id,)

        data_dict = {'id': catalog_id + '_' + dataset_id}
        package = self.portal.call_action('package_show', data_dict=data_dict)
        self.assertEqual(return_id, catalog_id + '_' + dataset_id)
        self.assertEqual('updated description', package['notes'])

    @CKAN_VCR.use_cassette()
    def test_resources_swapped_correctly(self):
        catalog_id = 'same-catalog-id'
        full_dataset = self.full_catalog.datasets[0]
        full_dataset_id = full_dataset['identifier']
        push_dataset_to_ckan(self.full_catalog, 'oficina-de-muestra', full_dataset_id,
                             self.portal_url, self.apikey, catalog_id=catalog_id,)

        justice_dataset = self.justice_catalog.datasets[0]
        justice_dataset_id = justice_dataset['identifier']
        push_dataset_to_ckan(self.justice_catalog, 'oficina-de-muestra', justice_dataset_id,
                             self.portal_url, self.apikey, catalog_id=catalog_id,)
        # Switch them and update
        full_dataset['distribution'], justice_dataset['distribution'] = \
            justice_dataset['distribution'], full_dataset['distribution']

        full_package_id = push_dataset_to_ckan(self.full_catalog,'oficina-de-muestra', full_dataset_id,
                                               self.portal_url, self.apikey, catalog_id=catalog_id,)
        justice_package_id = push_dataset_to_ckan(self.justice_catalog, 'oficina-de-muestra',
                                                  justice_dataset_id, self.portal_url,
                                                  self.apikey, catalog_id=catalog_id,)
        # Switch them back
        full_dataset['distribution'], justice_dataset['distribution'] = \
            justice_dataset['distribution'], full_dataset['distribution']

        data_dict = {'id': full_package_id}
        full_package = self.portal.call_action(
            'package_show', data_dict=data_dict)
        data_dict = {'id': justice_package_id}
        justice_package = self.portal.call_action(
            'package_show', data_dict=data_dict)

        self.assertEqual(len(full_package['resources']), len(
            justice_dataset['distribution']))
        self.assertEqual(len(justice_package['resources']), len(
            full_dataset['distribution']))

        for resource, justice_distribution in zip(full_package['resources'], justice_dataset['distribution']):
            self.assertEqual('same-catalog-id_' +
                             justice_distribution['identifier'], resource['id'])

        for resource, full_distribution in zip(justice_package['resources'], full_dataset['distribution']):
            self.assertEqual('same-catalog-id_' +
                             full_distribution['identifier'], resource['id'])


class RemoveTestCase(unittest.TestCase):
    CKAN_VCR = vcr.VCR(path_transformer=vcr.VCR.ensure_suffix('.yaml'),
                       cassette_library_dir=os.path.join(
                           "tests", "cassetes", "ckan_integration", "remove_dataset"),
                       filter_headers=['Authorization', 'X-CKAN-API-Key'],
                       record_mode='once')

    test_datasets = [{'id': '1.1', 'owner_org': 'org-1', 'author': 'author_a', 'name': 'data1_1'},
                     {'id': '2.1', 'owner_org': 'org-2',
                         'author': 'author_a', 'name': 'data2_1'},
                     {'id': '2.2', 'owner_org': 'org-2',
                         'author': 'author_b', 'name': 'data2_2'},
                     {'id': '3.1', 'owner_org': 'org-3',
                         'author': 'author_a', 'name': 'data3_1'},
                     {'id': '3.2', 'owner_org': 'org-3',
                         'author': 'author_b', 'name': 'data3_2'},
                     {'id': '3.3', 'owner_org': 'org-3', 'author': 'author_c', 'name': 'data3_3'}]

    portal_url = 'http://localhost:8080'
    apikey = "<apikey>"

    @CKAN_VCR.use_cassette()
    def setUp(self):
        self.ckan_portal = RemoteCKAN(self.portal_url, apikey=self.apikey)
        for dataset in self.test_datasets:
            try:
                self.ckan_portal.call_action(
                    'dataset_purge', data_dict={'id': dataset['id']})
            except NotFound:
                continue
        for dataset in self.test_datasets:
            self.ckan_portal.call_action('package_create', data_dict=dataset)

    @CKAN_VCR.use_cassette()
    def tearDown(self):
        for dataset in self.test_datasets:
            try:
                self.ckan_portal.call_action(
                    'dataset_purge', data_dict={'id': dataset['id']})
            except NotFound:
                continue

    @CKAN_VCR.use_cassette()
    def test_remove_dataset_by_id(self):
        filter_in = {'dataset': {'identifier': '1.1'}}
        remove_datasets_from_ckan(
            self.portal_url, self.apikey, filter_in=filter_in)
        package_list = self.ckan_portal.call_action('package_list')
        self.assertTrue('data1_1' not in package_list)

    @CKAN_VCR.use_cassette()
    def test_remove_dataset_by_title(self):
        filter_in = {'dataset': {'title': 'data3_3'}}
        remove_datasets_from_ckan(
            self.portal_url, self.apikey, filter_in=filter_in)
        package_list = self.ckan_portal.call_action('package_list')
        self.assertTrue('data3_3' not in package_list)

    @CKAN_VCR.use_cassette()
    def test_remove_dataset_by_organization(self):
        remove_datasets_from_ckan(
            self.portal_url, self.apikey, organization='org-2')
        package_list = self.ckan_portal.call_action('package_list')
        self.assertTrue('data2_1' not in package_list)
        self.assertTrue('data2_2' not in package_list)

    @CKAN_VCR.use_cassette()
    def test_remove_dataset_by_publisher_and_organization(self):
        filter_in = {'dataset': {'publisher': {
            'name': 'author_b', 'mbox': None}}}
        remove_datasets_from_ckan(
            self.portal_url, self.apikey, filter_in=filter_in, organization='org-3')
        package_list = self.ckan_portal.call_action('package_list')
        self.assertTrue('data3_2' not in package_list)

    @CKAN_VCR.use_cassette()
    def test_remove_dataset_by_filter_out(self):
        filter_out = {'dataset': {'publisher': {
            'name': 'author_b', 'mbox': None}}}
        remove_datasets_from_ckan(
            self.portal_url, self.apikey, filter_out=filter_out)
        package_list = self.ckan_portal.call_action('package_list')
        self.assertTrue('data2_2' in package_list)
        self.assertTrue('data3_2' in package_list)
        self.assertTrue('data2_1' not in package_list)
        self.assertTrue('data3_3' not in package_list)

    @CKAN_VCR.use_cassette()
    def test_remove_dataset_by_filter_out_and_organization(self):
        filter_out = {'dataset': {'publisher': {
            'name': 'author_b', 'mbox': None}}}
        remove_datasets_from_ckan(
            self.portal_url, self.apikey, filter_out=filter_out, organization='org-3')
        package_list = self.ckan_portal.call_action('package_list')
        self.assertTrue('data3_1' not in package_list)
        self.assertTrue('data3_3' not in package_list)

    @CKAN_VCR.use_cassette()
    def test_empty_query_result(self):
        filter_in = {'dataset': {'identifier': '4.4'}}
        package_list_pre = self.ckan_portal.call_action('package_list')
        remove_datasets_from_ckan(
            self.portal_url, self.apikey, filter_in=filter_in, organization='org-4')
        package_list_post = self.ckan_portal.call_action('package_list')
        self.assertEqual(len(package_list_pre), len(package_list_post))

    @CKAN_VCR.use_cassette()
    def test_with_no_parametres(self):
        package_list_pre = self.ckan_portal.call_action('package_list')

        remove_datasets_from_ckan(self.portal_url, self.apikey)

        package_list_post = self.ckan_portal.call_action('package_list')
        self.assertEqual(len(package_list_pre), len(package_list_post))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
import json
import os

from six import wraps

RESULTS_DIR = os.path.join("tests", "results")


def load_expected_result():
    def case_decorator(test):
        case_filename = test.__name__.split("test_")[-1]

        @wraps(test)
        def decorated_test(*args, **kwargs):
            result_path = os.path.join(RESULTS_DIR, case_filename + ".json")

            with io.open(result_path, encoding='utf8') as result_file:
                expected_result = json.load(result_file)

            kwargs["expected_result"] = expected_result
            test(*args, **kwargs)

        return decorated_test

    return case_decorator



# -*- coding: utf-8 -*-


from __future__ import unicode_literals

from tests.support.utils import jsonschema_str


def gen_error(catalog_error, dataset_error):
    return {
        "status": "ERROR",
        "error": {
            "catalog": {
                "status": "ERROR",
                "errors": [
                    catalog_error
                ],
                "title": "Datos Argentina"
            },
            "dataset": [
                {
                    "status": "ERROR",
                    "identifier": "99db6631-d1c9-470b-a73e-c62daa32c420",
                    "list_index": 0,
                    "errors": [
                        dataset_error,
                    ],
                    "title": "Sistema de contrataciones electrónicas"
                }
            ]
        }
    }


def multiple_missing_descriptions():
    return gen_error({
        "instance": None,
        "validator": "required",
        "path": [],
        "message": "%s is a required property" % jsonschema_str('description'),
        "error_code": 1,
        "validator_value": [
            "dataset",
            "title",
            "description",
            "publisher",
            "superThemeTaxonomy"
        ]
    }, {
        "instance": None,
        "validator": "required",
        "path": [
            "dataset",
            0
        ],
        "message": "%s is a required property" % jsonschema_str('description'),
        "error_code": 1,
        "validator_value": [
            "title",
            "description",
            "publisher",
            "superTheme",
            "distribution",
            "accrualPeriodicity",
            "issued"
        ]
    })


def invalid_multiple_fields_type():
    return gen_error({
        "instance": [
            "Ministerio de Modernización",
            "datos@modernizacion.gob.ar"
        ],
        "validator": "type",
        "path": [
            "publisher"
        ],
        "message": "[%s, %s] is not of type %s" % (
            jsonschema_str('Ministerio de Modernización'),
            jsonschema_str('datos@modernizacion.gob.ar'),
            jsonschema_str('object'),
        ),
        "error_code": 2,
        "validator_value": "object"
    }, {
        "instance": "5120",
        "validator": "anyOf",
        "path": [
            "dataset",
            0,
            "distribution",
            0,
            "byteSize"
        ],
        "message": "%s is not valid under any of the given schemas" % jsonschema_str(
            '5120'),
        "error_code": 2,
        "validator_value": [
            {
                "type": "integer"
            },
            {
                "type": "null"
            },
            {
                "type": "string",
                "maxLength": 0
            }
        ]
    })



# -*- coding: utf-8 -*-

"""Módulo 'download' de pydatajson

Contiene métodos para descargar archivos a través del protocolo HTTP.
"""

from __future__ import unicode_literals, print_function, with_statement
from __future__ import absolute_import

import requests
import time
import sys

DEFAULT_TRIES = 1
RETRY_DELAY = 1


def download(url, tries=DEFAULT_TRIES, retry_delay=RETRY_DELAY,
             try_timeout=None, proxies=None, verify=True):
    """
    Descarga un archivo a través del protocolo HTTP, en uno o más intentos.

    Args:
        url (str): URL (schema HTTP) del archivo a descargar.
        tries (int): Intentos a realizar (default: 1).
        retry_delay (int o float): Tiempo a esperar, en segundos, entre cada
            intento.
        try_timeout (int o float): Tiempo máximo a esperar por intento.
        proxies (dict): Proxies a utilizar. El diccionario debe contener los
            valores 'http' y 'https', cada uno asociados a la URL del proxy
            correspondiente.

    Returns:
        bytes: Contenido del archivo
    """
    for i in range(tries):
        try:
            return requests.get(url, timeout=try_timeout, proxies=proxies,
                                verify=verify).content
        except Exception as e:
            download_exception = e

            if i < tries - 1:
                time.sleep(retry_delay)

    raise download_exception


def download_to_file(url, file_path, **kwargs):
    """
    Descarga un archivo a través del protocolo HTTP, en uno o más intentos, y
    escribe el contenido descargado el el path especificado.

    Args:
        url (str): URL (schema HTTP) del archivo a descargar.
        file_path (str): Path del archivo a escribir. Si un archivo ya existe
            en el path especificado, se sobrescribirá con nuevos contenidos.
        kwargs: Parámetros para download().
    """
    content = download(url, **kwargs)
    with open(file_path, "wb") as f:
        f.write(content)


if __name__ == '__main__':
    download_to_file(sys.argv[1], sys.argv[2])



# -*- coding: utf-8 -*-


from __future__ import unicode_literals, absolute_import

from .catalog_errors import missing_catalog_title, \
    missing_catalog_description, \
    missing_catalog_dataset, invalid_catalog_publisher_type, invalid_publisher_mbox_format, \
    null_catalog_publisher, empty_mandatory_string, malformed_date, malformed_datetime, \
    malformed_datetime2, malformed_email, malformed_uri, invalid_theme_taxonomy, missing_dataset, \
    repeated_downloadURL
from .dataset_errors import missing_dataset_title, \
    missing_dataset_description, \
    malformed_accrualperiodicity, malformed_temporal, malformed_temporal2, too_long_field_title
from .distribution_errors import missing_distribution_title
from .other_errors import multiple_missing_descriptions, \
    invalid_multiple_fields_type

FULL_DATA_RESPONSE = {
    "status": "OK",
    "error": {
        "catalog": {
            "status": "OK",
            "errors": [],
            "title": "Datos Argentina"
        },
        "dataset": [
            {
                "status": "OK",
                "identifier": "99db6631-d1c9-470b-a73e-c62daa32c777",
                "list_index": 0,
                "errors": [],
                "title": "Sistema de contrataciones electrónicas"
            },
            {
                "status": "OK",
                "identifier": "99db6631-d1c9-470b-a73e-c62daa32c420",
                "list_index": 1,
                "errors": [],
                "title": "Sistema de contrataciones electrónicas (sin datos)"
            }
        ]
    }
}

TEST_FROM_RESULT_FILE = {
    # Tests de CAMPOS REQUERIDOS
    # Tests de inputs válidos
    'full_data': FULL_DATA_RESPONSE,
    # Un datajson con valores correctos únicamente para las claves requeridas
    'minimum_data': None,

    # Tests de TIPOS DE CAMPOS
    # Tests de inputs válidos
    'null_dataset_theme': None,
    'null_field_description': None,
    # Tests de inputs inválidos
    'invalid_catalog_publisher_type': None,
    'invalid_publisher_mbox_format': None,
    # Catalog_publisher y distribution_bytesize fallan
    'invalid_field_description_type': None,
    # La clave requerida catalog["description"] NO puede ser str vacía
    'empty_optional_string': None,
    # El format y extension de fileName de las distribuciones deben coincidir si estan los campos presentes
    'mismatched_fileName_and_format': None,
    # El format y extension de downloadURL de las distribuciones deben coincidir si estan los campos presentes
    'mismatched_downloadURL_and_format': None,
}

TEST_FROM_GENERATED_RESULT = {

    'multiple_missing_descriptions': multiple_missing_descriptions(),
    'invalid_multiple_fields_type': invalid_multiple_fields_type(),

    'missing_catalog_title': missing_catalog_title(),
    'missing_catalog_description': missing_catalog_description(),
    'missing_catalog_dataset': missing_catalog_dataset(),
    'null_catalog_publisher': null_catalog_publisher(),
    'empty_mandatory_string': empty_mandatory_string(),
    'malformed_datetime': malformed_datetime(),
    'malformed_datetime2': malformed_datetime2(),
    'malformed_email': malformed_email(),
    'malformed_uri': malformed_uri(),
    'invalid_themeTaxonomy': invalid_theme_taxonomy(),
    'missing_dataset': missing_dataset(),

    'missing_dataset_title': missing_dataset_title(),
    'missing_dataset_description': missing_dataset_description(),
    'malformed_accrualperiodicity': malformed_accrualperiodicity(),
    'malformed_date': malformed_date(),
    'malformed_temporal': malformed_temporal(),
    'malformed_temporal2': malformed_temporal2(),
    'too_long_field_title': too_long_field_title(),

    'missing_distribution_title': missing_distribution_title(),

    'invalid_catalog_publisher_type': invalid_catalog_publisher_type(),
    'invalid_publisher_mbox_format': invalid_publisher_mbox_format(),

    'repeated_downloadURL': repeated_downloadURL(),
}

TEST_FILE_RESPONSES = {}
TEST_FILE_RESPONSES.update(TEST_FROM_RESULT_FILE)
TEST_FILE_RESPONSES.update(TEST_FROM_GENERATED_RESULT)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Extensión de pydatajson para la federación de metadatos de datasets a través
de la API de CKAN.
"""

from __future__ import print_function
from ckanapi import RemoteCKAN
from ckanapi.errors import NotFound
from .ckan_utils import map_dataset_to_package, map_theme_to_group
from .search import get_datasets


def push_dataset_to_ckan(catalog, owner_org, dataset_origin_identifier,
                         portal_url, apikey, catalog_id=None,
                         demote_superThemes=True, demote_themes=True):
    """Escribe la metadata de un dataset en el portal pasado por parámetro.

        Args:
            catalog (DataJson): El catálogo de origen que contiene el dataset.
            owner_org (str): La organización a la cual pertence el dataset.
            dataset_origin_identifier (str): El id del dataset que se va a
                federar.
            portal_url (str): La URL del portal CKAN de destino.
            apikey (str): La apikey de un usuario con los permisos que le
                permitan crear o actualizar el dataset.
            catalog_id (str): El prefijo con el que va a preceder el id del
                dataset en catálogo destino.
            demote_superThemes(bool): Si está en true, los ids de los super
                themes del dataset, se propagan como grupo.
            demote_themes(bool): Si está en true, los labels de los themes
                del dataset, pasan a ser tags. Sino, se pasan como grupo.
        Returns:
            str: El id del dataset en el catálogo de destino.
    """
    dataset = catalog.get_dataset(dataset_origin_identifier)
    ckan_portal = RemoteCKAN(portal_url, apikey=apikey)

    package = map_dataset_to_package(catalog, dataset, owner_org, catalog_id,
                                     demote_superThemes, demote_themes)

    # Get license id
    if dataset.get('license'):
        license_list = ckan_portal.call_action('license_list')
        try:
            ckan_license = next(
                license_item for license_item in license_list if
                license_item['title'] == dataset['license'] or
                license_item['url'] == dataset['license']
            )
            package['license_id'] = ckan_license['id']
        except StopIteration:
            package['license_id'] = 'notspecified'
    else:
        package['license_id'] = 'notspecified'

    try:
        pushed_package = ckan_portal.call_action(
            'package_update', data_dict=package)
    except NotFound:
        pushed_package = ckan_portal.call_action(
            'package_create', data_dict=package)

    ckan_portal.close()
    return pushed_package['id']


def remove_dataset_from_ckan(identifier, portal_url, apikey):
    ckan_portal = RemoteCKAN(portal_url, apikey=apikey)
    ckan_portal.call_action('dataset_purge', data_dict={'id': identifier})


def remove_datasets_from_ckan(portal_url, apikey, filter_in=None,
                              filter_out=None, only_time_series=False,
                              organization=None):
    """Borra un dataset en el portal pasado por parámetro.

            Args:
                portal_url (str): La URL del portal CKAN de destino.
                apikey (str): La apikey de un usuario con los permisos que le
                    permitan borrar el dataset.
                filter_in(dict): Diccionario de filtrado positivo, similar al
                    de search.get_datasets.
                filter_out(dict): Diccionario de filtrado negativo, similar al
                    de search.get_datasets.
                only_time_series(bool): Filtrar solo los datasets que tengan
                    recursos con series de tiempo.
                organization(str): Filtrar solo los datasets que pertenezcan a
                    cierta organizacion.
            Returns:
                None
    """
    ckan_portal = RemoteCKAN(portal_url, apikey=apikey)
    identifiers = []
    datajson_filters = filter_in or filter_out or only_time_series
    if datajson_filters:
        identifiers += get_datasets(
            portal_url + '/data.json',
            filter_in=filter_in, filter_out=filter_out,
            only_time_series=only_time_series, meta_field='identifier'
        )
    if organization:
        query = 'organization:"' + organization + '"'
        search_result = ckan_portal.call_action('package_search', data_dict={
                                                'q': query, 'rows': 500, 'start': 0})
        org_identifiers = [dataset['id']
                           for dataset in search_result['results']]
        start = 500
        while search_result['count'] > start:
            search_result = ckan_portal.call_action('package_search',
                                                    data_dict={'q': query, 'rows': 500, 'start': start})
            org_identifiers += [dataset['id']
                                for dataset in search_result['results']]
            start += 500

        if datajson_filters:
            identifiers = set(identifiers).intersection(set(org_identifiers))
        else:
            identifiers = org_identifiers

    for identifier in identifiers:
        ckan_portal.call_action('dataset_purge', data_dict={'id': identifier})


def push_theme_to_ckan(catalog, portal_url, apikey,
                       identifier=None, label=None):
    """Escribe la metadata de un theme en el portal pasado por parámetro.

            Args:
                catalog (DataJson): El catálogo de origen que contiene el
                    theme.
                portal_url (str): La URL del portal CKAN de destino.
                apikey (str): La apikey de un usuario con los permisos que le
                    permitan crear o actualizar el dataset.
                identifier (str): El identificador para buscar el theme en la
                    taxonomia.
                label (str): El label para buscar el theme en la taxonomia.
            Returns:
                str: El name del theme en el catálogo de destino.
        """
    ckan_portal = RemoteCKAN(portal_url, apikey=apikey)
    theme = catalog.get_theme(identifier=identifier, label=label)
    group = map_theme_to_group(theme)
    pushed_group = ckan_portal.call_action('group_create', data_dict=group)
    return pushed_group['name']


def restore_dataset_to_ckan(catalog, owner_org, dataset_origin_identifier,
                            portal_url, apikey):
    """Restaura la metadata de un dataset en el portal pasado por parámetro.

        Args:
            catalog (DataJson): El catálogo de origen que contiene el dataset.
            owner_org (str): La organización a la cual pertence el dataset.
            dataset_origin_identifier (str): El id del dataset que se va a
                restaurar.
            portal_url (str): La URL del portal CKAN de destino.
            apikey (str): La apikey de un usuario con los permisos que le
                permitan crear o actualizar el dataset.
        Returns:
            str: El id del dataset restaurado.
    """
    return push_dataset_to_ckan(catalog, owner_org, dataset_origin_identifier,
                                portal_url, apikey, None, False, False)


def harvest_dataset_to_ckan(catalog, owner_org, dataset_origin_identifier,
                            portal_url, apikey, catalog_id):
    """Federa la metadata de un dataset en el portal pasado por parámetro.

        Args:
            catalog (DataJson): El catálogo de origen que contiene el dataset.
            owner_org (str): La organización a la cual pertence el dataset.
            dataset_origin_identifier (str): El id del dataset que se va a
                restaurar.
            portal_url (str): La URL del portal CKAN de destino.
            apikey (str): La apikey de un usuario con los permisos que le
                permitan crear o actualizar el dataset.
            catalog_id(str): El id que prep
        Returns:
            str: El id del dataset restaurado.
    """

    return push_dataset_to_ckan(catalog, owner_org, dataset_origin_identifier,
                                portal_url, apikey, catalog_id=catalog_id)


def restore_catalog_to_ckan(catalog, owner_org, portal_url, apikey,
                            dataset_list=None):
    """Restaura los datasets de un catálogo al portal pasado por parámetro.
        Si hay temas presentes en el DataJson que no están en el portal de
        CKAN, los genera.

        Args:
            catalog (DataJson): El catálogo de origen que se restaura.
            portal_url (str): La URL del portal CKAN de destino.
            apikey (str): La apikey de un usuario con los permisos que le
                permitan crear o actualizar el dataset.
            dataset_list(list(str)): Los ids de los datasets a restaurar. Si no
                se pasa una lista, todos los datasests se restauran.
            owner_org (str): La organización a la cual pertencen los datasets.
                Si no se pasa, se utiliza el catalog_id.
        Returns:
            str: El id del dataset en el catálogo de destino.
    """
    push_new_themes(catalog, portal_url, apikey)
    dataset_list = dataset_list or [ds['identifier']
                                    for ds in catalog.datasets]
    restored = []
    for dataset_id in dataset_list:
        restored_id = restore_dataset_to_ckan(
            catalog, owner_org, dataset_id, portal_url, apikey)
        restored.append(restored_id)
    return restored


def harvest_catalog_to_ckan(catalog, portal_url, apikey, catalog_id,
                            dataset_list=None, owner_org=None):
    """Federa los datasets de un catálogo al portal pasado por parámetro.

        Args:
            catalog (DataJson): El catálogo de origen que se federa.
            portal_url (str): La URL del portal CKAN de destino.
            apikey (str): La apikey de un usuario con los permisos que le
                permitan crear o actualizar el dataset.
            catalog_id (str): El prefijo con el que va a preceder el id del
                dataset en catálogo destino.
            dataset_list(list(str)): Los ids de los datasets a federar. Si no
                se pasa una lista, todos los datasests se federan.
            owner_org (str): La organización a la cual pertencen los datasets.
                Si no se pasa, se utiliza el catalog_id.
        Returns:
            str: El id del dataset en el catálogo de destino.
    """
    dataset_list = dataset_list or [ds['identifier']
                                    for ds in catalog.datasets]
    owner_org = owner_org or catalog_id
    harvested = []
    for dataset_id in dataset_list:
        harvested_id = harvest_dataset_to_ckan(
            catalog, owner_org, dataset_id, portal_url, apikey, catalog_id)
        harvested.append(harvested_id)
    return harvested


def push_new_themes(catalog, portal_url, apikey):
    """Toma un catálogo y escribe los temas de la taxonomía que no están
    presentes.

        Args:
            catalog (DataJson): El catálogo de origen que contiene la
                taxonomía.
            portal_url (str): La URL del portal CKAN de destino.
            apikey (str): La apikey de un usuario con los permisos que le
                permitan crear o actualizar el dataset.
        Returns:
            str: Los ids de los temas creados.
    """
    ckan_portal = RemoteCKAN(portal_url, apikey=apikey)
    existing_themes = ckan_portal.call_action('group_list')
    new_themes = [theme['id'] for theme in catalog[
        'themeTaxonomy'] if theme['id'] not in existing_themes]
    pushed_names = []
    for new_theme in new_themes:
        name = push_theme_to_ckan(
            catalog, portal_url, apikey, identifier=new_theme)
        pushed_names.append(name)
    return pushed_names



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests del modulo pydatajson."""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import with_statement

import os.path
import unittest
import nose
from .context import pydatajson
from pydatajson.documentation import field_to_markdown
from pydatajson.documentation import distribution_to_markdown
from pydatajson.documentation import dataset_to_markdown


class DocumentationTestCase(unittest.TestCase):

    SAMPLES_DIR = os.path.join("tests", "samples")
    RESULTS_DIR = os.path.join("tests", "results")

    def test_field_to_markdown(self):
        field = {
            "title": "procedimiento_id",
            "type": "integer",
            "description": "Identificador único del procedimiento"
        }

        result = field_to_markdown(field)
        expected = "**procedimiento_id** (integer): Identificador único del procedimiento"
        self.assertEqual(result, expected)

        # elimino campo de type
        field.pop("type")
        result = field_to_markdown(field)
        expected = "**procedimiento_id**: Identificador único del procedimiento"
        self.assertEqual(result, expected)

        # elimino campo de description
        field.pop("description")
        result = field_to_markdown(field)
        expected = "**procedimiento_id**"
        self.assertEqual(result, expected)

    def test_distribution_to_markdown(self):
        distribution = {
            "title": "Convocatorias abiertas durante el año 2015",
            "description": "Listado de las convocatorias abiertas durante el año 2015 en el sistema de contrataciones electrónicas",
            "field": [
                {
                    "title": "procedimiento_id",
                    "type": "integer",
                    "description": "Identificador único del procedimiento"
                },
                {
                    "title": "unidad_operativa_contrataciones_id",
                    "type": "integer",
                    "description": "Identificador único de la unidad operativa de contrataciones"
                }
            ]
        }

        result = distribution_to_markdown(distribution)
        expected = """
### Convocatorias abiertas durante el año 2015

Listado de las convocatorias abiertas durante el año 2015 en el sistema de contrataciones electrónicas

#### Campos del recurso

- **procedimiento_id** (integer): Identificador único del procedimiento
- **unidad_operativa_contrataciones_id** (integer): Identificador único de la unidad operativa de contrataciones
"""
        self.assertEqual(result, expected)

        # elimino campos no obligatiorios
        distribution.pop("field")
        distribution.pop("description")
        result = distribution_to_markdown(distribution)
        expected = """
### Convocatorias abiertas durante el año 2015



#### Campos del recurso
"""
        print(result)
        self.assertEqual(result.strip(), expected.strip())

    def test_dataset_to_markdown(self):
        dataset = {
            "title": "Sistema de contrataciones electrónicas",
            "description": "Datos correspondientes al Sistema de Contrataciones Electrónicas (Argentina Compra)",
            "distribution": [
                {
                    "title": "Convocatorias abiertas durante el año 2015",
                    "description": "Listado de las convocatorias abiertas durante el año 2015 en el sistema de contrataciones electrónicas",
                    "field": [
                        {
                            "title": "procedimiento_id",
                            "type": "integer",
                            "description": "Identificador único del procedimiento"
                        },
                        {
                            "title": "unidad_operativa_contrataciones_id",
                            "type": "integer",
                            "description": "Identificador único de la unidad operativa de contrataciones"
                        }
                    ]
                },
                {
                    "title": "Convocatorias abiertas durante el año 2016",
                    "description": "Listado de las convocatorias abiertas durante el año 2016 en el sistema de contrataciones electrónicas",
                }
            ]
        }

        result = dataset_to_markdown(dataset)
        expected = """
# Sistema de contrataciones electrónicas

Datos correspondientes al Sistema de Contrataciones Electrónicas (Argentina Compra)

## Recursos del dataset


### Convocatorias abiertas durante el año 2015

Listado de las convocatorias abiertas durante el año 2015 en el sistema de contrataciones electrónicas

#### Campos del recurso

- **procedimiento_id** (integer): Identificador único del procedimiento
- **unidad_operativa_contrataciones_id** (integer): Identificador único de la unidad operativa de contrataciones

### Convocatorias abiertas durante el año 2016

Listado de las convocatorias abiertas durante el año 2016 en el sistema de contrataciones electrónicas

#### Campos del recurso
"""
        self.assertEqual(result.strip(), expected.strip())


if __name__ == '__main__':
    nose.run(defaultTest=__name__)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from tests.support.utils import jsonschema_str


def catalog_error(options=None):
    default_options = {
        'title': "Datos Argentina",
        'message': None,
        "validator": "required",
        "path": [],
        "instance": None,
        'dataset': [
            {
                "status": "OK",
                "identifier": "99db6631-d1c9-470b-a73e-c62daa32c420",
                "list_index": 0,
                "errors": [],
                "title": "Sistema de contrataciones electrónicas"
            }
        ],
        "error_code": 1,
        "validator_value": [
            "dataset",
            "title",
            "description",
            "publisher",
            "superThemeTaxonomy"
        ]
    }

    if options is not None:
        default_options.update(options)
    options = default_options
    return {
        "status": "ERROR",
        "error": {
            "catalog": {
                "status": "ERROR",
                "errors": [
                    {
                        "instance": options['instance'],
                        "validator": options['validator'],
                        "path": options['path'],
                        "message": options['message'],
                        "error_code": options["error_code"],
                        "validator_value": options['validator_value'],
                    }
                ],
                "title": options['title'],
            },
            "dataset": options['dataset'],
        }
    }


def missing_catalog_dataset():
    return catalog_error({
        'message': "%s is a required property" % jsonschema_str('dataset'),
        'dataset': None,
    })


def missing_catalog_title():
    return catalog_error({
        'message': "%s is a required property" % jsonschema_str('title'),
        'title': None,
    })


def missing_catalog_description():
    return catalog_error({
        'message': "%s is a required property" % jsonschema_str('description'),
    })


def invalid_catalog_publisher_type():
    return catalog_error({
        'instance': [
            "Ministerio de Modernización",
            "datos@modernizacion.gob.ar"
        ],
        "validator": "type",
        "path": ["publisher"],

        "message": "[%s, %s] is not of type %s" % (
            jsonschema_str('Ministerio de Modernización'),
            jsonschema_str('datos@modernizacion.gob.ar'),
            jsonschema_str('object'),
        ),
        "error_code": 2,
        "validator_value": "object"
    })


def invalid_publisher_mbox_format():
    return catalog_error({
        "error_code": 2,
        "message": "%s is not a %s" % (
            jsonschema_str('datosATmodernizacion.gob.ar'), jsonschema_str('email')),
        "validator": "format",
        "validator_value": "email",
        "path": [
            "publisher",
            "mbox"
        ],
        "instance": "datosATmodernizacion.gob.ar"
    })


def null_catalog_publisher():
    return catalog_error({
        "error_code": 2,
        "message": "None is not of type %s" % jsonschema_str('object'),
        "path": ['publisher'],
        "validator": 'type',
        "validator_value": 'object',
    })


def empty_mandatory_string():
    return catalog_error({
        "error_code": 2,
        "message": "%s is too short" % jsonschema_str(''),
        "path": ['description'],
        "validator": 'minLength',
        "validator_value": 1,
        "instance": "",
    })


def malformed_date():
    return catalog_error({
        "instance": "2016/04/14",
        "validator": "anyOf",
        "path": [
            "issued"
        ],
        "message": "%s is not valid under any of the given schemas" % jsonschema_str('2016/04/14'),
        "error_code": 2,
        "validator_value": [
            {
                "type": "string",
                "format": "date"
            },
            {
                "type": "string",
                "format": "date-time"
            },
            {
                "type": "null"
            },
            {
                "type": "string",
                "maxLength": 0
            }
        ]
    })


def malformed_datetime():
    return catalog_error({
        "instance": "2016-04-1419:48:05.433640-03:00",
        "validator": "anyOf",
        "path": [
            "issued"
        ],
        "message": "%s is not valid under any of the given schemas" % jsonschema_str(
            '2016-04-1419:48:05.433640-03:00'),
        "error_code": 2,
        "validator_value": [
            {
                "type": "string",
                "format": "date"
            },
            {
                "type": "string",
                "format": "date-time"
            },
            {
                "type": "null"
            },
            {
                "type": "string",
                "maxLength": 0
            }
        ]
    })


def malformed_datetime2():
    return catalog_error({
        "instance": "2016-04-54T19:48:05.433640-03:00",
        "validator": "anyOf",
        "path": [
            "issued"
        ],
        "message": "%s is not valid under any of the given schemas" % jsonschema_str(
            '2016-04-54T19:48:05.433640-03:00'),
        "error_code": 2,
        "validator_value": [
            {
                "type": "string",
                "format": "date"
            },
            {
                "type": "string",
                "format": "date-time"
            },
            {
                "type": "null"
            },
            {
                "type": "string",
                "maxLength": 0
            }
        ]
    })


def malformed_email():
    return catalog_error({
        "instance": "datosATmodernizacion.gob.ar",
        "validator": "format",
        "path": [
            "publisher",
            "mbox"
        ],
        "message": "%s is not a %s" % (
            jsonschema_str('datosATmodernizacion.gob.ar'), jsonschema_str('email')),
        "error_code": 2,
        "validator_value": "email"
    })


def malformed_uri():
    return catalog_error({
        "instance": "datos.gob.ar/superThemeTaxonomy.json",
        "validator": "format",
        "path": [
            "superThemeTaxonomy"
        ],
        "message": "%s is not a %s" % (
            jsonschema_str('datos.gob.ar/superThemeTaxonomy.json'), jsonschema_str('uri')),
        "error_code": 2,
        "validator_value": "uri"
    })


def invalid_theme_taxonomy():
    return catalog_error({
        "instance": None,
        "validator": "repeatedValue",
        "path": [
            "catalog",
            "themeTaxonomy"
        ],
        "message": "Los ids [%s] estan repetidos en mas de un `theme`" % jsonschema_str(
            'convocatorias'),
        "error_code": 2,
        "validator_value": "Chequea ids duplicados en themeTaxonomy"
    })


def missing_dataset():
    return catalog_error({
        "instance": None,
        "validator": "required",
        "path": [],
        "message": "%s is a required property" % jsonschema_str('dataset'),
        "error_code": 1,
        "validator_value": [
            "dataset",
            "title",
            "description",
            "publisher",
            "superThemeTaxonomy"
        ],
        "dataset": None,
    })


def repeated_downloadURL():
    return catalog_error({
        "instance": None,
        "validator": "repeatedValue",
        "path": [
            "catalog",
            "dataset"
        ],
        "message": "DownloadURL's [%s] estan repetidas en mas de un `distribution`" % jsonschema_str(
            'http://186.33.211.253/dataset/99db6631-d1c9-470b-a73e-c62daa32c420/resource/4b7447cb-31ff-4352-96c3-589d212e1cc9/download/convocatorias-abiertas-anio-2015.csv'),
        "error_code": 2,
        "validator_value": "Chequea downloadURL's duplicados en las distribuciones"
    })




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Módulo 'indicators' de Pydatajson

Contiene los métodos para monitorear y generar indicadores de un catálogo o de
una red de catálogos.
"""

from __future__ import print_function, absolute_import, unicode_literals, with_statement

import json
import os
from datetime import datetime

from six import string_types

from . import helpers
from . import readers
from .reporting import generate_datasets_summary

CENTRAL_CATALOG = "http://datos.gob.ar/data.json"
ABSOLUTE_PROJECT_DIR = os.path.dirname(os.path.abspath(__file__))
CATALOG_FIELDS_PATH = os.path.join(ABSOLUTE_PROJECT_DIR, "fields")


def generate_catalogs_indicators(catalogs, central_catalog=None,
                                 validator=None):
    """Genera una lista de diccionarios con varios indicadores sobre
    los catálogos provistos, tales como la cantidad de datasets válidos,
    días desde su última fecha actualizada, entre otros.

    Args:
        catalogs (str o list): uno o más catalogos sobre los que se quiera
            obtener indicadores
        central_catalog (str): catálogo central sobre el cual comparar los
            datasets subidos en la lista anterior. De no pasarse no se
            generarán indicadores de federación de datasets.

    Returns:
        tuple: 2 elementos, el primero una lista de diccionarios con los
            indicadores esperados, uno por catálogo pasado, y el segundo
            un diccionario con indicadores a nivel global,
            datos sobre la lista entera en general.
    """
    central_catalog = central_catalog or CENTRAL_CATALOG
    assert isinstance(catalogs, string_types + (dict, list))
    # Si se pasa un único catálogo, genero una lista que lo contenga
    if isinstance(catalogs, string_types + (dict,)):
        catalogs = [catalogs]

    # Leo todos los catálogos
    catalogs = [readers.read_catalog(catalog) for catalog in catalogs]

    indicators_list = []
    # Cuenta la cantidad de campos usados/recomendados a nivel global
    fields = {}
    for catalog in catalogs:
        catalog = readers.read_catalog(catalog)

        fields_count, result = _generate_indicators(
            catalog, validator=validator)
        if central_catalog:
            result.update(_federation_indicators(catalog,
                                                 central_catalog))

        indicators_list.append(result)
        # Sumo a la cuenta total de campos usados/totales
        fields = helpers.add_dicts(fields_count, fields)

    # Indicadores de la red entera
    network_indicators = {
        'catalogos_cant': len(catalogs)
    }

    # Sumo los indicadores individuales al total
    indicators_total = indicators_list[0].copy()
    for i in range(1, len(indicators_list)):
        indicators_total = helpers.add_dicts(indicators_total,
                                             indicators_list[i])
    network_indicators.update(indicators_total)
    # Genero los indicadores de la red entera,
    _network_indicator_percentages(fields, network_indicators)

    return indicators_list, network_indicators


def _generate_indicators(catalog, validator=None):
    """Genera los indicadores de un catálogo individual.

    Args:
        catalog (dict): diccionario de un data.json parseado

    Returns:
        dict: diccionario con los indicadores del catálogo provisto
    """
    result = {}
    # Obtengo summary para los indicadores del estado de los metadatos
    result.update(_generate_status_indicators(catalog, validator=validator))
    # Genero los indicadores relacionados con fechas, y los agrego
    result.update(_generate_date_indicators(catalog))
    # Agrego la cuenta de los formatos de las distribuciones
    count = _count_distribution_formats(catalog)
    result.update({
        'distribuciones_formatos_cant': count
    })
    # Agrego porcentaje de campos recomendados/optativos usados
    fields_count = _count_required_and_optional_fields(catalog)
    recomendados_pct = 100 * float(fields_count['recomendado']) / \
        fields_count['total_recomendado']
    optativos_pct = 100 * float(fields_count['optativo']) / \
        fields_count['total_optativo']
    result.update({
        'campos_recomendados_pct': round(recomendados_pct, 2),
        'campos_optativos_pct': round(optativos_pct, 2)
    })
    return fields_count, result


def _federation_indicators(catalog, central_catalog):
    """Cuenta la cantidad de datasets incluídos tanto en la lista
    'catalogs' como en el catálogo central, y genera indicadores a partir
    de esa información.

    Args:
        catalog (dict): catálogo ya parseado
        central_catalog (str o dict): ruta a catálogo central, o un dict
            con el catálogo ya parseado
    """
    central_catalog = readers.read_catalog(central_catalog)
    federados = 0  # En ambos catálogos
    no_federados = 0
    datasets_federados_eliminados_cant = 0
    datasets_federados = []
    datasets_no_federados = []
    datasets_federados_eliminados = []

    # busca c/dataset del catálogo específico a ver si está en el central
    for dataset in catalog.get('dataset', []):
        found = False
        for central_dataset in central_catalog.get('dataset', []):
            if datasets_equal(dataset, central_dataset):
                found = True
                federados += 1
                datasets_federados.append((dataset.get('title'),
                                           dataset.get('landingPage')))
                break
        if not found:
            no_federados += 1
            datasets_no_federados.append((dataset.get('title'),
                                          dataset.get('landingPage')))

    # busca c/dataset del central cuyo publisher podría pertenecer al
    # catálogo específico, a ver si está en el catálogo específico
    # si no está, probablemente signifique que fue eliminado
    filtered_central = _filter_by_likely_publisher(
        central_catalog.get('dataset', []),
        catalog.get('dataset', [])
    )
    for central_dataset in filtered_central:
        found = False
        for dataset in catalog.get('dataset', []):
            if datasets_equal(dataset, central_dataset):
                found = True
                break
        if not found:
            datasets_federados_eliminados_cant += 1
            datasets_federados_eliminados.append(
                (central_dataset.get('title'),
                 central_dataset.get('landingPage'))
            )

    if federados or no_federados:  # Evita división por 0
        federados_pct = 100 * float(federados) / (federados + no_federados)
    else:
        federados_pct = 0

    result = {
        'datasets_federados_cant': federados,
        'datasets_no_federados_cant': no_federados,
        'datasets_federados_eliminados_cant': datasets_federados_eliminados_cant,
        'datasets_federados_eliminados': datasets_federados_eliminados,
        'datasets_no_federados': datasets_no_federados,
        'datasets_federados': datasets_federados,
        'datasets_federados_pct': round(federados_pct, 2)
    }
    return result


def _network_indicator_percentages(fields, network_indicators):
    """Encapsula el cálculo de indicadores de porcentaje (de errores,
    de campos recomendados/optativos utilizados, de datasets actualizados)
    sobre la red de nodos entera.

    Args:
        fields (dict): Diccionario con claves 'recomendado', 'optativo',
            'total_recomendado', 'total_optativo', cada uno con valores
            que representan la cantidad de c/u en la red de nodos entera.

        network_indicators (dict): Diccionario de la red de nodos, con
            las cantidades de datasets_meta_ok y datasets_(des)actualizados
            calculados previamente. Se modificará este argumento con los
            nuevos indicadores.
    """

    # Los porcentuales no se pueden sumar, tienen que ser recalculados

    # % de datasets cuya metadata está ok
    meta_ok = network_indicators['datasets_meta_ok_cant']
    meta_error = network_indicators['datasets_meta_error_cant']
    total_pct = 0.0
    if meta_ok or meta_error:  # Evita división por cero
        total_pct = 100 * float(meta_ok) / (meta_error + meta_ok)
    network_indicators['datasets_meta_ok_pct'] = round(total_pct, 2)

    # % de campos recomendados y optativos utilizados en todo el catálogo
    if fields:  # 'fields' puede estar vacío si ningún campo es válido
        rec_pct = 100 * float(fields['recomendado']) / \
            fields['total_recomendado']

        opt_pct = 100 * float(fields['optativo']) / \
            fields['total_optativo']

        network_indicators.update({
            'campos_recomendados_pct': round(rec_pct, 2),
            'campos_optativos_pct': round(opt_pct, 2)
        })

    # % de datasets actualizados
    act = network_indicators['datasets_actualizados_cant']
    desact = network_indicators['datasets_desactualizados_cant']
    updated_pct = 0
    if act or desact:  # Evita división por cero
        updated_pct = 100 * act / float(act + desact)
    network_indicators['datasets_actualizados_pct'] = round(updated_pct, 2)

    # % de datasets federados
    federados = network_indicators.get('datasets_federados_cant')
    no_federados = network_indicators.get('datasets_no_federados_cant')

    if federados or no_federados:
        federados_pct = 100 * float(federados) / (federados + no_federados)
        network_indicators['datasets_federados_pct'] = \
            round(federados_pct, 2)


def _generate_status_indicators(catalog, validator=None):
    """Genera indicadores básicos sobre el estado de un catálogo

    Args:
        catalog (dict): diccionario de un data.json parseado

    Returns:
        dict: indicadores básicos sobre el catálogo, tal como la cantidad
        de datasets, distribuciones y número de errores
    """
    summary = generate_datasets_summary(catalog, validator=validator)
    cant_ok = 0
    cant_error = 0
    cant_distribuciones = 0
    datasets_total = len(summary)
    for dataset in summary:
        cant_distribuciones += dataset['cant_distribuciones']

        if dataset['estado_metadatos'] == "OK":
            cant_ok += 1
        else:  # == "ERROR"
            cant_error += 1

    datasets_ok_pct = 0
    if datasets_total:
        datasets_ok_pct = round(100 * float(cant_ok) / datasets_total, 2)
    result = {
        'datasets_cant': datasets_total,
        'distribuciones_cant': cant_distribuciones,
        'datasets_meta_ok_cant': cant_ok,
        'datasets_meta_error_cant': cant_error,
        'datasets_meta_ok_pct': datasets_ok_pct
    }
    return result


def _generate_date_indicators(catalog, tolerance=0.2):
    """Genera indicadores relacionados a las fechas de publicación
    y actualización del catálogo pasado por parámetro. La evaluación de si
    un catálogo se encuentra actualizado o no tiene un porcentaje de
    tolerancia hasta que se lo considere como tal, dado por el parámetro
    tolerance.

    Args:
        catalog (dict o str): path de un catálogo en formatos aceptados,
            o un diccionario de python

        tolerance (float): porcentaje de tolerancia hasta que se considere
            un catálogo como desactualizado, por ejemplo un catálogo con
            período de actualización de 10 días se lo considera como
            desactualizado a partir de los 12 con una tolerancia del 20%.
            También acepta valores negativos.

    Returns:
        dict: diccionario con indicadores
    """
    result = {}

    dias_ultima_actualizacion = _days_from_last_update(
        catalog, "modified")
    if not dias_ultima_actualizacion:
        dias_ultima_actualizacion = _days_from_last_update(
            catalog, "issued")

    result['catalogo_ultima_actualizacion_dias'] = \
        dias_ultima_actualizacion

    actualizados = 0
    desactualizados = 0
    periodicity_amount = {}

    for dataset in catalog.get('dataset', []):
        # Parseo la fecha de publicación, y la frecuencia de actualización
        periodicity = dataset.get('accrualPeriodicity')
        if not periodicity:
            continue
        # Si la periodicity es eventual, se considera como actualizado
        if periodicity == 'eventual':
            actualizados += 1
            prev_periodicity = periodicity_amount.get(periodicity, 0)
            periodicity_amount[periodicity] = prev_periodicity + 1
            continue

        # dataset sin fecha de última actualización es desactualizado
        if "modified" not in dataset:
            desactualizados += 1
        else:
            # Calculo el período de días que puede pasar sin actualizarse
            # Se parsea el período especificado por accrualPeriodicity,
            # cumple con el estándar ISO 8601 para tiempos con repetición
            date = helpers.parse_date_string(dataset['modified'])
            days_diff = float((datetime.now() - date).days)
            interval = helpers.parse_repeating_time_interval(
                periodicity) * \
                (1 + tolerance)

            if days_diff < interval:
                actualizados += 1
            else:
                desactualizados += 1

        prev_periodicity = periodicity_amount.get(periodicity, 0)
        periodicity_amount[periodicity] = prev_periodicity + 1

    datasets_total = len(catalog.get('dataset', []))
    actualizados_pct = 0
    if datasets_total:
        actualizados_pct = float(actualizados) / datasets_total
    result.update({
        'datasets_desactualizados_cant': desactualizados,
        'datasets_actualizados_cant': actualizados,
        'datasets_actualizados_pct': 100 * round(actualizados_pct, 2),
        'datasets_frecuencia_cant': periodicity_amount
    })
    return result


def _count_distribution_formats(catalog):
    """Cuenta los formatos especificados por el campo 'format' de cada
    distribución de un catálogo o de un dataset.

    Args:
        catalog (str o dict): path a un catálogo, o un dict de python que

    Returns:
        dict: diccionario con los formatos de las distribuciones
        encontradas como claves, con la cantidad de ellos en sus valores.
    """

    # Leo catálogo
    catalog = readers.read_catalog(catalog)
    catalog_formats = {}

    for dataset in catalog.get('dataset', []):
        dataset_formats = _count_distribution_formats_dataset(dataset)

        for distribution_format in dataset_formats:
            count_catalog = catalog_formats.get(distribution_format, 0)
            count_dataset = dataset_formats.get(distribution_format, 0)
            catalog_formats[
                distribution_format] = count_catalog + count_dataset

    return catalog_formats


def _count_distribution_formats_dataset(dataset):
    formats = {}
    for distribution in dataset['distribution']:
        # 'format' es recomendado, no obligatorio. Puede no estar.
        distribution_format = distribution.get('format', None)

        if distribution_format:
            # Si no está en el diccionario, devuelvo 0
            count = formats.get(distribution_format, 0)

            formats[distribution_format] = count + 1

    return formats


def _days_from_last_update(catalog, date_field="modified"):
    """Calcula días desde la última actualización del catálogo.

    Args:
        catalog (dict): Un catálogo.
        date_field (str): Campo de metadatos a utilizar para considerar
            los días desde la última actualización del catálogo.

    Returns:
        int or None: Cantidad de días desde la última actualización del
            catálogo o None, si no pudo ser calculada.
    """

    # el "date_field" se busca primero a nivel catálogo, luego a nivel
    # de cada dataset, y nos quedamos con el que sea más reciente
    date_modified = catalog.get(date_field, None)
    dias_ultima_actualizacion = None
    # "date_field" a nivel de catálogo puede no ser obligatorio,
    # si no está pasamos
    if isinstance(date_modified, string_types):
        date = helpers.parse_date_string(date_modified)
        dias_ultima_actualizacion = (
            datetime.now() - date).days if date else None

    for dataset in catalog.get('dataset', []):
        date = helpers.parse_date_string(dataset.get(date_field, ""))
        days_diff = float((datetime.now() - date).days) if date else None

        # Actualizo el indicador de días de actualización si corresponde
        if not dias_ultima_actualizacion or \
                (days_diff and days_diff < dias_ultima_actualizacion):
            dias_ultima_actualizacion = days_diff

    if dias_ultima_actualizacion:
        return int(dias_ultima_actualizacion)
    else:
        return None


def _count_required_and_optional_fields(catalog):
    """Cuenta los campos obligatorios/recomendados/requeridos usados en
    'catalog', junto con la cantidad máxima de dichos campos.

    Args:
        catalog (str o dict): path a un catálogo, o un dict de python que
            contenga a un catálogo ya leído

    Returns:
        dict: diccionario con las claves 'recomendado', 'optativo',
            'requerido', 'recomendado_total', 'optativo_total',
            'requerido_total', con la cantidad como valores.
    """

    catalog = readers.read_catalog(catalog)

    # Archivo .json con el uso de cada campo. Lo cargamos a un dict
    catalog_fields_path = os.path.join(CATALOG_FIELDS_PATH,
                                       'fields.json')
    with open(catalog_fields_path) as f:
        catalog_fields = json.load(f)

    # Armado recursivo del resultado
    return _count_fields_recursive(catalog, catalog_fields)


def _count_fields_recursive(dataset, fields):
    """Cuenta la información de campos optativos/recomendados/requeridos
    desde 'fields', y cuenta la ocurrencia de los mismos en 'dataset'.

    Args:
        dataset (dict): diccionario con claves a ser verificadas.
        fields (dict): diccionario con los campos a verificar en dataset
            como claves, y 'optativo', 'recomendado', o 'requerido' como
            valores. Puede tener objetios anidados pero no arrays.

    Returns:
        dict: diccionario con las claves 'recomendado', 'optativo',
            'requerido', 'recomendado_total', 'optativo_total',
            'requerido_total', con la cantidad como valores.
    """

    key_count = {
        'recomendado': 0,
        'optativo': 0,
        'requerido': 0,
        'total_optativo': 0,
        'total_recomendado': 0,
        'total_requerido': 0
    }

    for k, v in fields.items():
        # Si la clave es un diccionario se implementa recursivamente el
        # mismo algoritmo
        if isinstance(v, dict):
            # dataset[k] puede ser o un dict o una lista, ej 'dataset' es
            # list, 'publisher' no. Si no es lista, lo metemos en una.
            # Si no es ninguno de los dos, dataset[k] es inválido
            # y se pasa un diccionario vacío para poder comparar
            elements = dataset.get(k)
            if not isinstance(elements, (list, dict)):
                elements = [{}]

            if isinstance(elements, dict):
                elements = [dataset[k].copy()]
            for element in elements:
                # Llamada recursiva y suma del resultado al nuestro
                result = _count_fields_recursive(element, v)
                for key in result:
                    key_count[key] += result[key]
        # Es un elemento normal (no iterable), se verifica si está en
        # dataset o no. Se suma 1 siempre al total de su tipo
        else:
            # total_requerido, total_recomendado, o total_optativo
            key_count['total_' + v] += 1

            if k in dataset:
                key_count[v] += 1

    return key_count


def datasets_equal(dataset, other, fields_dataset=None,
                   fields_distribution=None, return_diff=False):
    """Función de igualdad de dos datasets: se consideran iguales si
    los valores de los campos 'title', 'publisher.name',
    'accrualPeriodicity' e 'issued' son iguales en ambos.

    Args:
        dataset (dict): un dataset, generado por la lectura de un catálogo
        other (dict): idem anterior

    Returns:
        bool: True si son iguales, False en caso contrario
    """
    dataset_is_equal = True
    dataset_diff = []

    # Campos a comparar. Si es un campo anidado escribirlo como lista
    if not fields_dataset:
        fields_dataset = [
            'title',
            ['publisher', 'name']
        ]

    for field_dataset in fields_dataset:
        if isinstance(field_dataset, list):
            value = helpers.traverse_dict(dataset, field_dataset)
            other_value = helpers.traverse_dict(other, field_dataset)
        else:
            value = dataset.get(field_dataset)
            other_value = other.get(field_dataset)

        if value != other_value:
            dataset_diff.append({
                "error_location": field_dataset,
                "dataset_value": value,
                "other_value": other_value
            })
            dataset_is_equal = False

    if fields_distribution:
        dataset_distributions = dataset.get("distribution")
        other_distributions = other.get("distribution")

        if len(dataset_distributions) != len(other_distributions):
            print("{} distribuciones en origen y {} en destino".format(
                len(dataset_distributions), len(other_distributions)))
            dataset_is_equal = False

        distributions_equal = True
        for dataset_distribution, other_distribution in zip(
                dataset_distributions, other_distributions):

            for field_distribution in fields_distribution:
                if isinstance(field_distribution, list):
                    value = helpers.traverse_dict(
                        dataset_distribution, field_distribution)
                    other_value = helpers.traverse_dict(
                        other_distribution, field_distribution)
                else:
                    value = dataset_distribution.get(field_distribution)
                    other_value = other_distribution.get(field_distribution)

                if value != other_value:
                    dataset_diff.append({
                        "error_location": "{} ({})".format(
                            field_distribution,
                            dataset_distribution.get("title")
                        ),
                        "dataset_value": value,
                        "other_value": other_value
                    })
                    distributions_equal = False

        if not distributions_equal:
            dataset_is_equal = False

    if return_diff:
        return dataset_diff
    else:
        return dataset_is_equal


def _filter_by_likely_publisher(central_datasets, catalog_datasets):
    publisher_names = [
        catalog_dataset["publisher"]["name"]
        for catalog_dataset in catalog_datasets
        if "name" in catalog_dataset["publisher"]
    ]

    filtered_central_datasets = []
    for central_dataset in central_datasets:
        if "name" in central_dataset["publisher"] and \
                central_dataset["publisher"]["name"] in publisher_names:
            filtered_central_datasets.append(central_dataset)

    return filtered_central_datasets



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Módulo `time_series` de pydatajson

Contiene funciones auxiliares para analizar catálogos con series de tiempo,
definidas según la extensión del perfil de metadatos para series de tiempo.
"""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import with_statement
import os


def field_is_time_series(field, distribution=None):
    field_may_be_ts = (
        not field.get("specialType") and
        not field.get("specialTypeDetail") and
        (
            field.get("type", "").lower() == "number" or
            field.get("type", "").lower() == "integer"
        ) and
        field.get("id")
    )
    distribution_may_has_ts = (
        not distribution or distribution_has_time_index(distribution)
    )
    return field_may_be_ts and distribution_may_has_ts


def distribution_has_time_index(distribution):
    for field in distribution.get('field', []):
        if field.get('specialType') == 'time_index':
            return True
    return False


def dataset_has_time_series(dataset):
    for distribution in dataset.get('distribution', []):
        if distribution_has_time_index(distribution):
            return True
    return False
